<?php
/**
 * Plugin Name:       WP Zone Helper
 * Description:	      Get direct one-click access to download plugins and child themes, easily connect services, and contact support for WP Zone (including Aspen Grove Studios) – right from your WordPress dashboard.
 * Author:            WP Zone
 * Author URI:        https://wpzone.co/
 * Version:           1.0.37
 * Text Domain:       ags-dashboard
 * License:           GNU General Public License version 3 or later
 * License URI:       https://www.gnu.org/licenses/gpl-3.0.en.html
 * GitLab Plugin URI: https://gitlab.com/aspengrovestudios/ags-ds-dashboard/
 * AGS Info:          ids.divispace 1064409
 */

/*
WP Zone Helper
Copyright (C) 2024  WP Zone

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

============

This plugin contains code from the Easy Digital Downloads Software Licensing addon.
Copyright (c) Sandhills Development, LLC
Released under the GNU General Public License (GPL) version 2 or
later, licensed under GPL version 3 or later (see license/license.txt).

This plugin includes code based on parts of WordPress by Automattic, released
under GPLv2+, licensed under GPLv3 or later (see wp-license.txt in the license directory
for the license and additional credits applicable to WordPress, and the license.txt
file in the license directory for GPLv3 text).
*/

if (!class_exists('AGSDashboard')) {
	
class AGSDashboard {

	const	VERSION = '1.0.37',
			OWN_PRODUCT_ID = ['divispace' => 1064409],
			OWN_LICENSE_KEY = 'Ac5e4a9dbfe8c6670e080bf59a2565af',
			PRODUCT_TYPES = ['theme', 'plugin'],
			SITE_URL = 'https://wpzone.co/',
			USER_PREFS = [
				'keysVisibleToOthers' => 0
			],
			SUPPORT_EMAIL = 'support@aspengrovestudios.com';
	
	public static 	$pluginName,
					$pluginBaseUrl,
					$pluginDirectory,
					$pluginFile,
					$products,
					$currentUserAccount,
					$productsWithUpdaters = [],
					$productsExtraFields = [],
					$productCatalog,
					$licenseKeys,
					$currentUserLicenseKeys,
					$mergedCatalog,
					$isSelfDeactivating,
					$filterLegacyOptions = true,
					$hasAdminNotice = false,
					$lastLicenseKeyErrorMessage = null;
	
	public static function setup() {
		self::$pluginName = __('WP Zone Helper', 'ags-dashboard');
		self::$pluginBaseUrl = plugin_dir_url(__FILE__);
		self::$pluginDirectory = __DIR__.'/';
		self::$pluginFile = __FILE__;
		
		
		add_filter( 'pre_set_site_transient_update_plugins', array( __CLASS__, 'checkUpdatesPlugins' ) );
		add_filter( 'pre_set_site_transient_update_themes', array( __CLASS__, 'checkUpdatesThemes' ) );
		
		add_filter( 'plugins_api', array( __CLASS__, 'filterPluginsApi' ), 10, 3 );
		
		add_action( 'after_plugin_row', array( __CLASS__, 'showUpdateNotification' ), 9, 2 );
		add_action( 'admin_init', array( __CLASS__, 'showChangelog' ) );
		
		// wp-layouts\ags-layouts.php
		add_action('admin_menu', array(__CLASS__, 'registerAdminPages'), 11);
		add_action('network_admin_menu', array(__CLASS__, 'registerAdminPages'), 11);
		add_action('admin_notices', array(__CLASS__, 'adminNotices'));
		add_action('admin_enqueue_scripts', array(__CLASS__, 'adminScripts'));
		
		add_action('wp_ajax_ags_dashboard_license_key_form', [__CLASS__, 'licenseKeyFormAjax']);
		add_action('wp_ajax_ags_dashboard_edit_license_key', [__CLASS__, 'editLicenseKeyAjax']);
		add_action('wp_ajax_ags_dashboard_change_active_status', [__CLASS__, 'changeActiveStatusAjax']);
		add_action('wp_ajax_ags_dashboard_wizard', [__CLASS__, 'wizardAjax']);
		add_action('wp_ajax_ags_dashboard_install', [__CLASS__, 'installAjax']);
		add_action('wp_ajax_ags_dashboard_get_support_articles', [__CLASS__, 'getSupportArticlesAjax']);
		add_action('wp_ajax_ags_dashboard_check_updates', [__CLASS__, 'checkUpdatesAjax']);
		add_action('wp_ajax_ags_dashboard_refresh_catalog', [__CLASS__, 'refreshCatalogAjax']);
		add_action('wp_ajax_ags_dashboard_check_license_keys', [__CLASS__, 'checkLicenseKeysAjax']);
		add_action('wp_ajax_ags_dashboard_set_user_preference', [__CLASS__, 'setUserPreferenceAjax']);
		add_action('wp_ajax_ags_dashboard_admin_notice_dismiss', [__CLASS__, 'dismissAdminNoticeAjax']);
		add_action('wp_ajax_ags_dashboard_set_remote_access', [__CLASS__, 'setRemoteAccessAjax']);
		add_action('wp_ajax_ags_dashboard_open_support_ticket', [__CLASS__, 'openSupportTicketAjax']);

		add_filter( 'upgrader_pre_download', [__CLASS__, 'maybeHandleUpdateDownload'], 10, 3 );
		
		
		add_action('update_site_option_active_sitewide_plugins', [__CLASS__, 'generateLegacySupportPlugin'] );
		add_action('update_option_active_plugins', [__CLASS__, 'generateLegacySupportPlugin'] );
		add_action('switch_theme', [__CLASS__, 'generateLegacySupportPlugin'] );
		
		add_action('ags_dashboard_check_license_keys', [__CLASS__, 'checkLicenseKeys']);
		add_action('ags_dashboard_delete_remote_user', [__CLASS__, 'deleteRemoteAccessUser']);
		
		// phpcs:ignore WordPress.Security.NonceVerification -- nonce will be checked in the action
		if (isset($_POST['ags_support_access'])) {
			add_action( 'after_setup_theme', array( __CLASS__, 'handleSupportAccess' ) );
		}
		
		if ( !get_site_option('ags_dashboard_wizard_shown') ) {
			add_action( 'admin_init', [__CLASS__, 'maybeRedirectToWizard'] );
		}
		
		add_action('set_current_user', [__CLASS__, 'checkRemoteAccess']);
		
	}
	
	static function checkRemoteAccess() {
		if ( wp_get_current_user()->user_email == 'support@aspengrovestudios.com' && get_option( 'ags_dashboard_remote_access_expiry', 0 ) <= time() ) {
			wp_logout();
			wp_die('Remote access has expired');
		}
	}
	
	static function maybeRedirectToWizard() {
		global $pagenow;
		// phpcs:ignore WordPress.Security.NonceVerification -- just a redirect
		if ( empty($pagenow) || $pagenow != 'admin.php' || empty($_GET['page']) || $_GET['page'] != 'ags-dashboard-settings' || empty($_GET['wizard']) ) {
			wp_safe_redirect(admin_url('admin.php?page=ags-dashboard-settings&wizard=1'));
			exit;
		}
	}
	
	static function wizardAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		// phpcs:ignore WordPress.Security.NonceVerification -- step is read-only
		if (!isset($_POST['step']) || ( $_POST['step'] != 1 && $_POST['step'] != 2 && $_POST['step'] != 3 ) ) {
			wp_send_json_error( __('Missing or invalid parameter: step', 'ags-dashboard') );
		}
		
		ob_start();
		
		// phpcs:ignore WordPress.Security.NonceVerification -- step is read-only
		include(__DIR__.'/includes/admin/wizard/step'.((int) $_POST['step']).'.php');
		
		$html = ob_get_clean();
		
		wp_send_json_success([
			'wizardHtml' => $html
		]);
	}
	
	// wp-layouts\ags-layouts.php
	public static function registerAdminPages() {
		/* Admin Pages */
		$mainPage = add_menu_page( esc_html(self::$pluginName), esc_html(self::$pluginName), 'manage_options', 'ags-dashboard-products', null, esc_html(self::$pluginBaseUrl .'images/svg/dashboard-logo-icon.svg'), 650 );
		add_submenu_page('ags-dashboard-products', esc_html__( 'Products', 'ags-dashboard'), esc_html__( 'Products', 'ags-dashboard'), 'manage_options', 'ags-dashboard-products', array(__CLASS__, 'productsPage'));
		if ( !is_network_admin() ) {
			add_submenu_page('ags-dashboard-products', esc_html__( 'Support', 'ags-dashboard'), esc_html__( 'Support', 'ags-dashboard'), 'manage_options', 'ags-dashboard-support', array(__CLASS__, 'supportPage'));
		}
		add_submenu_page('ags-dashboard-products', esc_html__( 'Settings', 'ags-dashboard'), esc_html__( 'Settings', 'ags-dashboard'), 'manage_options', 'ags-dashboard-settings', array(__CLASS__, 'settingsPage'));
	}
	
	public static function productsPage($forceIsNetworkAdmin=false) {
		include(self::$pluginDirectory.'includes/admin/pages/products.php');
	}
	
	public static function supportPage() {
		include(self::$pluginDirectory.'includes/admin/pages/support.php');
	}
	
	public static function settingsPage() {
		include(self::$pluginDirectory.'includes/admin/pages/settings.php');
	}
	
	
	// wp-layouts\ags-layouts.php
	public static function adminScripts() {
		// phpcs:ignore WordPress.Security.NonceVerification, ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- read-only operation, comparing against known value
		if ( isset($_GET['page']) && substr($_GET['page'], 0, 14) == 'ags-dashboard-' ) {
			wp_enqueue_style('ags-dashboard', self::$pluginBaseUrl.'css/admin.css', array(), self::VERSION);
			wp_enqueue_script('ags-dashboard', self::$pluginBaseUrl.'js/admin.js', array('jquery', 'wp-i18n'), self::VERSION);
			wp_set_script_translations('ags-dashboard', 'ags-dashboard', self::$pluginBaseUrl.'languages');
			//wp_enqueue_style('ags-dashboard-dialog', self::$pluginBaseUrl.'css/ags-dialog.css', array(), self::VERSION);
			wp_enqueue_script('ags-dashboard-dialog', self::$pluginBaseUrl.'js/ags-dialog.js', array('jquery', 'wp-i18n'), self::VERSION);
			wp_set_script_translations('ags-dashboard-dialog', 'ags-dashboard', self::$pluginBaseUrl.'languages');
		}
        wp_enqueue_style('ags-dashboard-admin-menu', self::$pluginBaseUrl.'css/admin-menu.css', array(), self::VERSION);
	}
	
	
	static function getInstalledProducts($extraFields=[]) {
		$nonCachedExtraFields = array_diff($extraFields, self::$productsExtraFields);
		if ( !isset(self::$products) || $nonCachedExtraFields ) {
			
			self::$products = [];
			$extraFields = array_merge(self::$productsExtraFields, $nonCachedExtraFields);
			
			require_once(ABSPATH.'wp-admin/includes/plugin.php');
			$allPluginFiles = array_keys( get_plugins() );
			$allThemeFiles = array_keys( wp_get_themes() );
			$themeRoot = get_theme_root();
			
			$headerNames = [
				'plugin' => [ 'info' => 'AGS Info' ],
				'theme' => [ 'info' => 'AGS Info' ]
			];
			
			if ( in_array('name', $extraFields) ) {
				$headerNames['plugin']['name'] = 'Plugin Name';
				$headerNames['theme']['name'] = 'Theme Name';
			}
			
			foreach (
				array_merge(
					array_combine( $allPluginFiles, array_fill( 0, count($allPluginFiles), 'plugin' ) ),
					array_combine( $allThemeFiles, array_fill( 0, count($allThemeFiles), 'theme' ) )
				)
				as $name => $productType
			) {
				
				$file = $productType == 'theme' ? $themeRoot.'/'.$name.'/style.css' : WP_PLUGIN_DIR.'/'.$name;
				$data = get_file_data( $file, $headerNames[$productType], $productType );
				
				if (!empty($data['info'])) {
					$info = explode(' ', $data['info']);
					
					$info = array_values( array_filter($info, function($infoItem) {
						return (bool) $infoItem;
					}) );
					
					$product = [];
					
					for ( $i = 0; $i < count($info); ) {
						
						$key = explode( '.', $info[$i++] );
						
						if (!isset($info[$i])) {
							break;
						}
						
						$value = $info[$i++];
						
						$keyCount = count($key);
						$current = &$product;
						
						foreach ($key as $j => $keyPart) {
							
							if ($j == $keyCount - 1) {
								$current[$keyPart] = $value;
								break;
							}
							if ( !isset($current[$keyPart]) ) {
								$current[$keyPart] = [];
							}
							$current = &$current[$keyPart];
							
						}
					}
					
					if ( !empty($product['ids']) ) {
						
						$product['file'] = $file;
						$product['type'] = $productType;
						
						switch ($productType) {
							case 'plugin':
								$product['active'] = is_plugin_active($name);
								break;
							case 'theme':
								$product['active'] = get_stylesheet() === $name;
								break;
						}
						
						if ($extraFields) {
							
							unset($catalogInfo);
							
							foreach (self::getMergedCatalog() as $catalogProduct) {
								if ( array_intersect_assoc( $product['ids'], $catalogProduct['ids']) ) {
									$catalogInfo = $catalogProduct;
									break;
								}
							}
							
							foreach ($extraFields as $field) {
								switch ($field) {
									case 'name':
										$product['name'] = isset($catalogInfo['name']) ? $catalogInfo['name'] : $data['name'];
										break;
									case 'url':
										$product['url'] = isset($catalogInfo['url']) ? $catalogInfo['url'] : '';
										break;
									case 'thumbnails':
										if ( isset($catalogInfo['thumbnails']) ) {
											$product['thumbnails'] = $catalogInfo['thumbnails'];
										}
										break;
								}
							}
							
						}
						
						self::$products[] = $product;
					}
				}
				
			}
			
			self::$productsExtraFields = $extraFields;
		}
		
		return self::$products;
	}
	
	
	
	
	static function getInstalledPluginBasenames() {
		return array_map( 'plugin_basename', array_column( self::getInstalledProducts(), 'file') );
	}
	
	static function getInstalledPluginSlugs() {
		$slugs = [];
		foreach ( array_column( self::getInstalledProducts(), 'file') as $file ) {
			$slugs[] = basename($file, '.php');
		}
		return $slugs;
	}
	
	static function getInstalledThemeSlugs() {
		return array_map( 'basename', array_map( 'dirname', array_column( self::getInstalledProducts(), 'file') ) );
	}
	
	static function getProductIdsBySlug($slug) {
		foreach ( array_column( self::getInstalledProducts(), 'ids', 'file') as $file => $ids ) {
			if ( basename($file, '.php') == $slug ) {
				return $ids;
			}
		}
	}
	
	static function getActiveThemeProductIds() {
		foreach ( self::getInstalledProducts() as $installedProduct ) {
			if ( $installedProduct['active'] && $installedProduct['type'] === 'theme'  ) {
				return $installedProduct['ids'];
			}
		}
	}
	
	static function slug2name($slug, $type) {
		switch ($type) {
			case 'plugin':
				foreach ( self::getInstalledPluginBasenames() as $name ) {
					if ( dirname($name) == $slug ) {
						return $name;
					}
				}
				break;
			case 'theme':
				return $slug.'/style.css';
		}
	}
	
	static function name2ids($name, $type) {
		foreach ( self::getInstalledProducts() as $product ) {
			if ( $product['type'] == $type ) {
				switch ($type) {
					case 'plugin':
						if ( plugin_basename( $product['file'] ) == $name ) {
							return $product['ids'];
						}
						break;
					case 'theme':
						if ( basename( dirname( $product['file'] ) ) == $name ) {
							return $product['ids'];
						}
						break;
				}
			}
		}
		
		return [];
	}
	
	
	static function getProductLicenseKeyInfo($productIds, $allowLegacy=true, $delete=false) {
		
		if (!isset(self::$licenseKeys)) {
			self::$licenseKeys = get_option('aspengrove_license_keys', []);
		}
		
		foreach ( $productIds as $site => $productId ) {
			if ( isset( self::$licenseKeys[$site.$productId] ) ) {
				$result = self::$licenseKeys[$site.$productId];
				if ($delete) {
					unset(self::$licenseKeys[$site.$productId]);
					update_option('aspengrove_license_keys', self::$licenseKeys);
				}
				return $result;
			}
		}
		
		if (!$delete) {
			
			if ($productIds == self::OWN_PRODUCT_ID) {
				return self::activateLicenseKey(self::OWN_PRODUCT_ID, self::OWN_LICENSE_KEY);
			}
			
			if ($allowLegacy) {
				self::$filterLegacyOptions = false;		
			
				$products = self::getInstalledProducts();
				
				foreach ( $products as $product ) {
					
					if ( isset($product['ids']) ) {
						
						foreach ( $product['ids'] as $site => $productId ) {
							
							if ( isset($productIds[$site]) && $productIds[$site] == $productId ) {
								
								if ( isset($product['legacy']['key']) && isset($product['legacy']['status']) ) {
									
									$key = self::getLegacyOption( $product['legacy']['key'] );
									$status = self::getLegacyOption( $product['legacy']['status'] );
											
									if ($key && $status) {
										
										// Delete option fields
										self::deleteLegacyOption( $product['legacy']['key'] );
										self::deleteLegacyOption( $product['legacy']['status'] );
										
										if ($status == 'valid') {
											unset($invalidKey);
											
											// We need to reactivate so that we get a signature
											$licenseKeyInfo = self::activateLicenseKey($productIds, $key, true);
											break;
										} else {
											$invalidKey = $key;
										}
									}
									
								}
								
								break;
							}
							
						}
						
					}
					
					
					
				}
				
				if ( !empty($licenseKeyInfo) ) {
					return $licenseKeyInfo;
				}
				
				
				if ( isset($invalidKey) ) {
					$licenseKeyInfo = [
						'key' => $key,
						'site' => $site,
						'expiration' => null,
						'status' => 'invalid',
						'signature' => null,
					];
					
					// Register the invalid license key
					self::registerLicenseKey(
						$site,
						$itemId,
						$licenseKeyInfo
					);
					
					return $licenseKeyInfo;
				}
				
				self::$filterLegacyOptions = true;		
			}
		
		}
		
		return [
			'key' => null,
			'site' => null,
			'expiration' => null,
			'status' => 'none',
			'signature' => null,
		];
		
	}
	
	static function getLegacyOption($option) {
		if (substr($option, 0, 3) != 'et.') {
			return get_option($option);
		}
		
		if (function_exists('et_get_option')) {
			return et_get_option(substr($option, 3));
		}
		
		return false;
	}
	
	static function setLegacyOption($option, $value) {
		if (substr($option, 0, 3) != 'et.') {
			if (self::$isSelfDeactivating) {
				remove_all_filters('sanitize_option_'.$option);
			}
			
			return update_option($option, $value);
		}
		
		if (function_exists('et_get_option')) {
			return et_update_option(substr($option, 3), $value);
		}
		
		return false;
	}
	
	static function deleteLegacyOption($option) {
		if (substr($option, 0, 3) != 'et.') {
			return delete_option($option);
		}
		
		if (function_exists('et_delete_option')) {
			return et_delete_option(substr($option, 3));
		}
		
		return false;
	}
	
	static function getProductCatalog() {
		
		if (!isset( self::$productCatalog )) {
			
			$productCatalog = is_network_admin() ? get_site_transient('divispace_product_catalog') : get_transient('divispace_product_catalog');
			if ($productCatalog) {
				$productCatalog = @json_decode($productCatalog, true);
			}
			
			if ( is_array($productCatalog) ) {
				self::$productCatalog = $productCatalog;
			} else {
				$productCatalog = self::requestJson('https://wpzone.co/product-catalog.json', [], true);
				
				if (is_array($productCatalog)) {
					$productCatalog = array_filter($productCatalog, function($catalogItem) {
						return !isset($catalogItem['hide']) || !in_array('helper', $catalogItem['hide']);
					});
					is_network_admin() ? set_site_transient( 'divispace_product_catalog', json_encode($productCatalog), 86400 ) : set_transient( 'divispace_product_catalog', json_encode($productCatalog), 86400 );
					self::$productCatalog = $productCatalog;
				} else {
					self::$productCatalog = [];
				}
			}
			
		}
		
		return self::$productCatalog;
		
	}
	
	static function getLicenseKeyStatusName($status, $expiry=null) {
		switch ($status) {
			case 'valid':
				if ($expiry == 'lifetime') {
					return __('Valid (lifetime)', 'ags-dashboard');
				}
				
				if ( $expiry >= time() ) {
					return sprintf( __('Valid until %s', 'ags-dashboard'), wp_date( get_option('date_format'), $expiry ) );
				}
				
				return sprintf( __('Expired on %s', 'ags-dashboard'), wp_date( get_option('date_format'), $expiry ) );
			case 'invalid':
				return __('Invalid', 'ags-dashboard');
			case 'none':
				return __('Not Activated', 'ags-dashboard');
		}
		return $status;
	}
	
	static function editLicenseKeyAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		if ( empty( $_POST['ags_dashboard_nonce'] ) || empty( $_POST['name'] ) || empty( $_POST['type'] )
				// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- nonce
				|| !wp_verify_nonce( $_POST['ags_dashboard_nonce'], 'ags_dashboard_change_key_'.( (int) !empty( $_POST['activate'] ) ).'_'.$_POST['name'] ) ) {
			wp_send_json_error( __('Request validation failed. Please refresh the page and try again.', 'ags-dashboard') );
		}
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- name2ids will compare against known values
		$productIds = self::name2ids( $_POST['name'], $_POST['type'] );
		if ( empty( $productIds ) ) {
			wp_send_json_error( __('Unknown product.', 'ags-dashboard') );
		}
		
		$licenseKeyInfo = self::getProductLicenseKeyInfo( $productIds );
		
		if ( empty($_POST['activate']) ) {
			if ( !empty($licenseKeyInfo['site']) && !empty($licenseKeyInfo['key']) && isset( $productIds[ $licenseKeyInfo['site'] ] ) ) {
				if ( !self::deactivateLicenseKey( $licenseKeyInfo['site'], $productIds[ $licenseKeyInfo['site'] ], $licenseKeyInfo['key'] ) ) {
					$errorMessage = empty(self::$lastLicenseKeyErrorMessage) ? __('Your license key could not be deactivated. Please contact support or try again.', 'ags-dashboard') :  self::$lastLicenseKeyErrorMessage;
				}
			} else {
				$errorMessage = __('Could not find an activated license key for a known product using the information provided.', 'ags-dashboard');
			}
			
			$licenseKeyInfo = self::getProductLicenseKeyInfo( $productIds );
		} else {
			
			if (!empty( $licenseKeyInfo['key'] )) {
				wp_send_json_error( __('Please deactivate the existing license key before activating a new one.', 'ags-dashboard') );
			}
			
			if (empty( $_POST['licenseKey'] )) {
				wp_send_json_error( __('Please specify a license key to activate.', 'ags-dashboard') );
			}
			
			// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- licenseKey will be validated during the activation process
			self::activateLicenseKey( $productIds, $_POST['licenseKey'] );
			
			$licenseKeyInfo = self::getProductLicenseKeyInfo( $productIds );
			
			if ( $licenseKeyInfo['status'] != 'valid' ) {
				$errorMessage = empty(self::$lastLicenseKeyErrorMessage) ? __('Your license key could not be activated. Please contact support or try again.', 'ags-dashboard') : self::$lastLicenseKeyErrorMessage;
			}
			
		}
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- type and name will be escaped on output
		$response = self::getLicenseKeyFormHtml( admin_url('admin-ajax.php'), $licenseKeyInfo, $_POST['type'], $_POST['name'] );
		
		$response['status'] = self::getLicenseKeyStatusName( $licenseKeyInfo['status'], $licenseKeyInfo['expiration'] );
		switch ($licenseKeyInfo['status']) {
			case 'valid':
				$response['statusClass'] = $licenseKeyInfo['expiration'] == 'lifetime' || $licenseKeyInfo['expiration'] > (time() + 86400) ? 'ags-dashboard-active' : 'ags-dashboard-warning';
				break;
			case 'none':
				break;
			default:
				$response['statusClass'] = 'ags-dashboard-error';
		}
		$response['rowClassRemovePrefix'] = 'ags-dashboard-key-status-';
		$response['rowClassAdd'] = 'ags-dashboard-key-status-'.$licenseKeyInfo['status'];
		
		if (isset($errorMessage)) {
			$response['error'] = $errorMessage;
		}
		
		wp_send_json_success($response);
	}
	
	static function licenseKeyFormAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
			
		// phpcs:ignore WordPress.Security.NonceVerification -- read-only operation
		if ( empty( $_GET['name'] ) || empty( $_GET['type'] ) ) {
			wp_send_json_error( __('Missing required field(s) "name" and/or "type".', 'ags-dashboard') );
		}
		
		// phpcs:ignore WordPress.Security.NonceVerification, ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- read-only operation; name2ids will compare against known values
		$productIds = self::name2ids( $_GET['name'], $_GET['type'] );
		if ( empty( $productIds ) ) {
			wp_send_json_error( __('Unknown product.', 'ags-dashboard') );
		}
		
		$licenseKeyInfo = self::getProductLicenseKeyInfo( $productIds );
		
		// phpcs:ignore WordPress.Security.NonceVerification, ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- read-only operation; type and name will be escaped on output
		$response = self::getLicenseKeyFormHtml( admin_url('admin-ajax.php'), $licenseKeyInfo, $_GET['type'], $_GET['name'] );
		
		wp_send_json_success($response);
	}
	
	
	static function getUserPreference($name, $userId=null) {
		if (!$userId) {
			$userId = get_current_user_id();
		}
		
		$prefs = get_user_meta($userId, 'ags_dashboard_prefs', true);
		if (!$prefs) {
			return isset(self::USER_PREFS[$name]) ? self::USER_PREFS[$name] : null;
		}
		
		$prefs = @json_decode($prefs);
		if ( !isset($prefs->$name) ) {
			return isset(self::USER_PREFS[$name]) ? self::USER_PREFS[$name] : null;
		}
		
		return $prefs->$name;
	}
	
	static function setUserPreference($name, $value) {
		
		if ( !isset( self::USER_PREFS[$name] ) ) {
			return false;
		}
		
		if ( is_int(self::USER_PREFS[$name]) ) {
			$value = (int) $value;
		}
		
		$userId = get_current_user_id();
		
		$prefs = get_user_meta($userId, 'ags_dashboard_prefs', true);
		
		if ($prefs) {
			$prefs = @json_decode($prefs, true);
			if ( !$prefs ) {
				return false;
			}
		} else {
			$prefs = [];
		}
		
		
		if ($value === self::USER_PREFS[$name]) {
			if (isset($prefs[$name])) {
				unset( $prefs[$name] );
			} else {
				return true; // no change needed
			}
		} else if ( !isset($prefs[$name]) || $prefs[$name] != $value ) {
			$prefs[$name] = $value;
		} else {
			return true; // no change needed
		}
		
		if ($prefs) {
			$prefsEncoded = json_encode( $prefs );
			if (!$prefsEncoded) {
				return false;
			}
			return update_user_meta( $userId, 'ags_dashboard_prefs', $prefsEncoded );
		} else {
			return delete_user_meta( $userId, 'ags_dashboard_prefs' );
		}
		
	}
	
	
	static function setUserPreferenceAjax() {
		if (
			empty( $_POST['nonce'] )
				// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- no need to sanitize the nonce
				|| !wp_verify_nonce( $_POST['nonce'], 'ags-dashboard-change-user-prefs' )
				|| empty( $_POST['pref'] )
				|| !isset( $_POST['value'] )
				// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- setUserPreference is responsible for sanitizing
				|| !self::setUserPreference( $_POST['pref'], $_POST['value'] )
		) {
			wp_send_json_error();
		}
		
		wp_send_json_success();
	}
	
	static function getLicenseKeyFormHtml($ajaxUrl, $licenseKeyInfo, $productType, $productName) {
		$keyIsVisible = !empty($licenseKeyInfo['owner'])
							&& (
								$licenseKeyInfo['owner'] === get_current_user_id()
								|| self::getUserPreference( 'keysVisibleToOthers', false, $licenseKeyInfo['owner'] )
							);
		
		ob_start();
?>
		
		<form action="<?php echo( esc_url( add_query_arg( 'action', 'ags_dashboard_edit_license_key', $ajaxUrl ) ) ); ?>">
            <div class="ags-dashboard-license-key-wrapper">
                <label>
                    <span><?php esc_html_e('License key:', 'ags-dashboard'); ?></span>

                    <input
                        type="text"
                        name="licenseKey"
                        <?php
                            if ( !empty( $licenseKeyInfo['key'] ) ) {
                                echo( 'value="'.esc_html( $keyIsVisible
                                            ? $licenseKeyInfo['key']
                                            : str_pad( substr($licenseKeyInfo['key'], -4), strlen($licenseKeyInfo['key']), '*', STR_PAD_LEFT) ).'"'
                                );
                            }
                        ?>
                        <?php
                            if ( !current_user_can('ags_dashboard_edit_license_keys') || !empty($licenseKeyInfo['key']) ) {
                                echo('readonly');
                            }
                        ?>
                    >
                </label>

                <?php
                    if ( current_user_can('ags_dashboard_edit_license_keys') && empty($licenseKeyInfo['key']) ) {
                            $selectLicenseKeys = self::getCurrentUserLicenseKeysByProductIds( self::name2ids($productName, $productType) );
                        if ($selectLicenseKeys) {
                ?>
                            <div class="ags-dashboard-license-key-dropdown ags-dashboard-dropdown">
                                <a href="#"><?php esc_html_e('Select License Key', 'ags-dashboard'); ?></a>
                                <ul>
                                <?php
                                    foreach ( $selectLicenseKeys as $selectLicenseKeyData ) {
                                        if ( $selectLicenseKeyData['status'] == 'active' || $selectLicenseKeyData['status'] == 'inactive' ) {
                                ?>
                                        <li>
                                            <a href="#">
                                                <span class="ags-dashboard-license-key-select-desc"><?php echo( esc_html( $selectLicenseKeyData['title'] ) ); ?>:</span>
                                                <span class="ags-dashboard-license-key-select-key"><?php echo( esc_html($selectLicenseKeyData['key']) ); ?></span>
                                            </a>
                                        </li>
                                <?php
                                        }
                                    }
                                ?>
                                </ul>
                            </div>
                <?php
                        }
                    }
                ?>
            </div>

			<?php if (!empty($licenseKeyInfo['key'])) { ?>
			<div class="ags-dashboard-key-status">
				<?php if ( !empty($licenseKeyInfo['checkError']) ) { ?>
					<span class="ags-dashboard-license-key-check-error ags-dashboard-tooltip-icon">
						<?php esc_html_e('Something went wrong while checking the expiry date of this license key, so this information may not be up to date. If this message persists and your license key is still valid, you can try deactivating and reactivating your license key to see if that resets this error.', 'ags-dashboard'); ?>
					</span>
				<?php } ?>
				<?php echo( esc_html( self::getLicenseKeyStatusName( $licenseKeyInfo['status'], isset($licenseKeyInfo['expiration']) ? $licenseKeyInfo['expiration'] : null ) ) ); ?>
			</div>
			<?php } ?>
			
			<input
				type="hidden"
				name="name"
				value="<?php echo( esc_attr( $productName ) ); ?>"
			>
			
			<input
				type="hidden"
				name="type"
				value="<?php echo( esc_attr( $productType ) ); ?>"
			>
			
			<input
				type="hidden"
				name="ags_dashboard_nonce"
				value="<?php echo( esc_attr( wp_create_nonce('ags_dashboard_change_key_'.( (int) empty($licenseKeyInfo['key']) ).'_'.$productName) ) ); ?>"
			>
			
		</form>
<?php
		$return = [
			'formHtml' => ob_get_clean()
		];
		if ( current_user_can('ags_dashboard_edit_license_keys') ) {
			$return['buttonText'] = empty($licenseKeyInfo['key']) ? __('Activate', 'ags-dashboard') : __('Deactivate', 'ags-dashboard');
			$return['buttonValue'] = (int) empty($licenseKeyInfo['key']);
		}
		return $return;
	}
	
	static function getProductVersion($file) {
		// Theme files end in .css
		$headerInfo = get_file_data($file, ['Version' => 'Version'], $file[strlen($file) - 1] === 's' ? 'theme' : 'plugin');
		return empty( $headerInfo['Version'] ) ? false : $headerInfo['Version'];
	}
	
	static function checkUpdatesPlugins($_transient_data) {
		return self::checkUpdates( $_transient_data, 'plugin' );
	}
	
	static function checkUpdatesThemes($_transient_data) {
		return self::checkUpdates( $_transient_data, 'theme' );
	}

	static function getVersionInfo() {
		
		$version_info = self::getCachedVersionInfo();

		if ( false === $version_info ) {

			$version_info = [];
			$products = self::getInstalledProducts();
			
			foreach ( $products as $product ) {
				
				if ( is_numeric( current( $product['ids'] ) )
						&& (
							in_array( plugin_basename($product['file']), self::$productsWithUpdaters )
							|| array_intersect_assoc( $product['ids'], self::OWN_PRODUCT_ID )
						)
				) {
					
			
					if ( is_multisite() ) {
						$licenseKeys = get_network_option('aspengrove_license_keys', []);
						$validKeys = [];
						foreach ( $product['ids'] as $site => $productId ) {
							if ( isset( $licenseKeys[$site.$productId] ) ) {
								foreach ( $licenseKeys[$site.$productId] as $licenseKey ) {
									if ( $licenseKey['expiration'] == 'lifetime' || $licenseKey['expiration'] > time() ) {
										$validKeys[] = [
											'key' => $licenseKey['key'],
											'site' => $site
										];
									}
								}
							}
						}
						if ($validKeys) {
							$licenseKeyInfo = $validKeys[ array_rand($validKeys) ];
						}
					} else {
						$licenseKeyInfo = self::getProductLicenseKeyInfo( $product['ids'] );
					}
					
					
					if (empty($licenseKeyInfo['key'])) {
						$productKey = '';
						reset($product['ids']);
						$productSite = key($product['ids']);
					} else {
						$productKey = $licenseKeyInfo['key'];
						$productSite = $licenseKeyInfo['site'];
					}
					
					$version_info[ $product['file'] ] = self::eddUpdatesApiRequest( $productSite, [
						'edd_action' => 'get_version',
						'license'    => $productKey,
						//'item_name'  => isset( $data['item_name'] ) ? $data['item_name'] : false,
						'item_id'    => $product['ids'][$productSite],
						'version'    => self::getProductVersion($product['file']),
						'slug'       => basename( dirname( $product['file'] ) ),
						'author'     => 'Aspen Grove Studios',
						'url'        => home_url(),
						//'beta'       => ! empty( $data['beta'] ),
					] );
					
				}
				
			}
			
			self::setCachedVersionInfo( $version_info );
		}
		
		return $version_info;
	}

	public static function checkUpdates( $_transient_data, $type ) {

		global $pagenow;
		
		if ( ! is_object( $_transient_data ) ) {
			$_transient_data = new stdClass;
		}
		
		if (!isset($_transient_data->response)) {
			$_transient_data->response = [];
		}
		
		if (!isset($_transient_data->checked)) {
			$_transient_data->checked = [];
		}


		if ( 'plugins.php' == $pagenow && is_multisite() && $type == 'plugin' ) {
			return $_transient_data;
		}
		
		$version_info = self::getVersionInfo();
		
		foreach ( $version_info as $productFile => $productInfo ) {
			
			if ( false !== $productInfo && is_object( $productInfo ) && isset( $productInfo->new_version ) ) {
				
				
				
				/*
				Get the $updateId and filter the product type all at once. Plugin files always end in .php
				(last character = p) while theme files always end in .css (last character = s) so we make sure
				that the last filename character matches the type of product we are currently checking for updates
				for.
				*/
				switch ( $productFile[strlen($productFile) - 1].$type ) {
					case 'pplugin':
						$updateId = plugin_basename($productFile);
						break;
					case 'stheme':
						$updateId = basename( dirname( $productFile ) );
						break;
					default:
						continue 2;
				}
				
				$currentVersion = self::getProductVersion($productFile);

				if ( version_compare( $currentVersion, $productInfo->new_version, '<' ) ) {
					$_transient_data->response[ $updateId ] = $productInfo;
				}
				
				$_transient_data->checked[ $updateId ] = $currentVersion;

			}
			
		}

		$_transient_data->last_checked           = current_time( 'timestamp' );
		
		return $_transient_data;
	}

	/**
	 * show update nofication row -- needed for multisite subsites, because WP won't tell you otherwise!
	 *
	 * @param string  $file
	 * @param array   $plugin
	 */
	public static function showUpdateNotification( $file, $plugin ) {

		if ( is_network_admin() ) {
			return;
		}
		
		if( ! current_user_can( 'update_plugins' ) ) {
			return;
		}

		if( ! is_multisite() ) {
			return;
		}
		
		$installedPluginBasenames = self::getInstalledPluginBasenames();

		if ( !in_array( $file, $installedPluginBasenames ) ) {
			return;
		}
		
		remove_action( 'after_plugin_row_' . $file, 'wp_plugin_update_row', 10 );

		$update_cache = get_site_transient( 'update_plugins' );
		$update_cache = is_object( $update_cache ) ? $update_cache : new stdClass();

		if ( empty( $update_cache->response ) || array_diff( $installedPluginBasenames, array_keys( $update_cache->response ) ) ) {

			$version_info = self::getVersionInfo();
			
			foreach ( $version_info as $productFile => $productInfo ) {
				
				$updateId = plugin_basename($productFile);
				if ( empty( $update_cache->response[ $updateId ] ) && false !== $productInfo && is_object( $productInfo ) && isset( $productInfo->new_version ) && $productFile[strlen($productFile) - 1] == 'p') {
					
					$currentVersion = self::getProductVersion($productFile);

					if ( version_compare( $currentVersion, $productInfo->new_version, '<' ) ) {
						$update_cache->response[ $updateId ] = $productInfo;
					}
					
					$update_cache->checked[ $updateId ] = $currentVersion;
				}
				
			}
			
			$update_cache->last_checked = current_time( 'timestamp' );
				
			// Remove our filter on the site transient
			remove_filter( 'pre_set_site_transient_update_plugins', array( __CLASS__, 'checkUpdatesPlugins' ), 10 );
			
			set_site_transient( 'update_plugins', $update_cache );
			
			// Restore our filter
			add_filter( 'pre_set_site_transient_update_plugins', array( __CLASS__, 'checkUpdatesPlugins' ) );
			
		}
		
		if ( empty( $update_cache->response[ $file ] ) ) {
			return;
		}

		$version_info = $update_cache->response[ $file ];
		
		$currentVersion = self::getProductVersion(WP_PLUGIN_DIR.'/'.$productFile);

		if ( version_compare( $currentVersion, $version_info->new_version, '<' ) ) {
			
			$slug = basename( $file, '.php' );
			
			// build a plugin list row, with update notification
			$wp_list_table = _get_list_table( 'WP_Plugins_List_Table' );
			# <tr class="plugin-update-tr"><td colspan="' . $wp_list_table->get_column_count() . '" class="plugin-update colspanchange">
			echo '<tr class="plugin-update-tr" id="' . esc_attr($slug) . '-update" data-slug="' . esc_attr($slug) . '" data-plugin="' . esc_attr($slug . '/' . $file) . '">';
			echo '<td colspan="3" class="plugin-update colspanchange">';
			echo '<div class="update-message notice inline notice-warning notice-alt">';

			$changelog_link = self_admin_url( 'index.php?ags_dashboard_action=view_plugin_changelog&plugin=' . rawurlencode($file) . '&slug=' . $slug . '&TB_iframe=true&width=772&height=911' );

			if ( empty( $version_info->download_link ) ) {
				printf(
					esc_html__( 'There is a new version of %1$s available. %2$sView version %3$s details%4$s.', 'ags-dashboard' ),
					esc_html( $version_info->name ),
					'<a target="_blank" class="thickbox" href="' . esc_url( $changelog_link ) . '">',
					esc_html( $version_info->new_version ),
					'</a>'
				);
			} else {
				printf(
					esc_html__( 'There is a new version of %1$s available. %2$sView version %3$s details%4$s or %5$supdate now%6$s.', 'ags-dashboard' ),
					esc_html( $version_info->name ),
					'<a target="_blank" class="thickbox" href="' . esc_url( $changelog_link ) . '">',
					esc_html( $version_info->new_version ),
					'</a>',
					'<a href="' . esc_url( wp_nonce_url( self_admin_url( 'update.php?action=upgrade-plugin&plugin=' ) . $file, 'upgrade-plugin_' . $file ) ) .'">',
					'</a>'
				);
			}

			do_action( "in_plugin_update_message-{$file}", $plugin, $version_info );

			echo '</div></td></tr>';
		}
	}

	/**
	 * Updates information on the "View version x.x details" page with custom data.
	 *
	 * @uses api_request()
	 *
	 * @param mixed   $_data
	 * @param string  $_action
	 * @param object  $_args
	 * @return object $_data
	 */
	public static function filterPluginsApi( $_data, $_action = '', $_args = null ) {

		if ( $_action != 'plugin_information' ) {
			return $_data;
		}

		if ( !isset( $_args->slug ) ) {
			return $_data;
		}

		$to_send = array(
			'slug'   => $_args->slug,
			'is_ssl' => is_ssl(),
			'fields' => array(
				'banners' => array(),
				'reviews' => false
			)
		);

		// Get the transient where we store the api request for this plugin for 24 hours
		$versionInfo = self::getVersionInfo();
		
		foreach ( $versionInfo as $file => $info ) {
			if ( basename( dirname($file) ) == $_args->slug ) {
				$itemData = $info;
				break;
			}
		}
		
		if ( !isset( $itemData ) ) {
			return $_data;
		}
		
		$_data = $itemData;
		
		// Convert sections into an associative array, since we're getting an object, but Core expects an array.
		if ( isset( $_data->sections ) && ! is_array( $_data->sections ) ) {
			$new_sections = array();
			foreach ( $_data->sections as $key => $value ) {
				$new_sections[ $key ] = $value;
			}

			$_data->sections = $new_sections;
		}

		// Convert banners into an associative array, since we're getting an object, but Core expects an array.
		if ( isset( $_data->banners ) && ! is_array( $_data->banners ) ) {
			$new_banners = array();
			foreach ( $_data->banners as $key => $value ) {
				$new_banners[ $key ] = $value;
			}

			$_data->banners = $new_banners;
		}

		return $_data;
	}

	/**
	 * Disable SSL verification in order to prevent download update failures
	 *
	 * @param array   $args
	 * @param string  $url
	 * @return object $array
	 */
	public function filterHttpRequestArgs( $args, $url ) {
		// If it is an https request and we are performing a package download, disable ssl verification
		if ( strpos( $url, 'https://' ) !== false && strpos( $url, 'edd_action=package_download' ) ) {
			$args['sslverify'] = false;
		}
		return $args;
	}

	private static function eddUpdatesApiRequest( $site, $data ) {

		global $wp_version;
		
		$updatesUrl = self::SITE_URL;

		if( $updatesUrl == trailingslashit (home_url() ) ) {
			return false; // Don't allow a plugin to ping itself
		}

		$api_params = $data;

		$request = self::requestJson( $updatesUrl, $api_params );

		if ( $request && isset( $request->sections ) ) {
			$request->sections = maybe_unserialize( $request->sections );
		} else {
			$request = false;
		}

		if ( $request && isset( $request->banners ) ) {
			$request->banners = maybe_unserialize( $request->banners );
		}

		if( ! empty( $request->sections ) ) {
			foreach( $request->sections as $key => $section ) {
				$request->$key = (array) $section;
			}
		}

		return $request;
	}
	
	private static function requestJson( $url, $requestData = [], $returnArray = false ) {
		$request = $requestData
						? wp_remote_post( $url, array( 'timeout' => 15, 'sslverify' => false, 'body' => $requestData ) )
						: wp_remote_get( $url, array( 'timeout' => 15, 'sslverify' => false ) );
		
		if ( ! is_wp_error( $request ) ) {
			return json_decode( wp_remote_retrieve_body( $request ), $returnArray );
		}
	}

	public static function showChangelog() {
		global $edd_plugin_data;
		
		// phpcs:ignore WordPress.Security.NonceVerification -- read-only operation
		if( empty( $_REQUEST['ags_dashboard_action'] ) || 'view_plugin_changelog' != $_REQUEST['ags_dashboard_action'] || empty( $_REQUEST['plugin'] ) || empty( $_REQUEST['slug'] ) ) {
			return;
		}

		if( ! current_user_can( 'update_plugins' ) ) {
			wp_die( esc_html__( 'You do not have permission to install plugin updates', 'ags-dashboard' ), esc_html__( 'Error', 'ags-dashboard' ), array( 'response' => 403 ) );
		}
		
		$versionInfo = self::getVersionInfo();
		
		foreach ($versionInfo as $file => $versionData) {
			// phpcs:ignore WordPress.Security.NonceVerification -- read-only operation
			if ( plugin_basename($file) == $_REQUEST['plugin'] && $versionData->sections->changelog ) {
				// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped -- HTML from server
				echo '<div style="background:#fff;padding:10px;">' . $versionData->sections->changelog . '</div>';
			}
		}
		
		exit;
	}

	public static function getCachedVersionInfo() {
		
		$cache = get_site_transient( 'aspengrove_updates' );

		if( empty( $cache ) ) {
			return false; // Cache is expired
		}

		return json_decode( $cache );

	}

	public static function setCachedVersionInfo( $value = '' ) {

		set_site_transient( 'aspengrove_updates', wp_json_encode( $value ), 3600 * 3 );

	}
	
	static function activateLicenseKey($productIds, $key, $fromLegacy=false) {
		
		if ($key) {
			if ( isset($productIds['divispace']) ) {
				$result = self::_activateLicenseKey('divispace', $productIds['divispace'], $key, $fromLegacy);
				if ($result) {
					return $result;
				}
			}
			
			if ( isset($productIds['aspengrove']) ) {
				return self::_activateLicenseKey('aspengrove', $productIds['aspengrove'], $key, $fromLegacy);
			}
		}
		
		
		
		
		return false;
	}
	
	private static function _activateLicenseKey($site, $itemId, $key, $fromLegacy) {
		
		// data to send in our API request
		$api_params = array(
			'edd_action' => 'activate_license',
			'license'    => $key,
			//'item_name'  => urlencode( ds_icon_expansion_ITEM_NAME ), // the name of our product in EDD
			'item_id' => $itemId,
			'url'        => home_url()
		);
		
		$license_data = self::requestJson(self::SITE_URL, $api_params);

		self::$lastLicenseKeyErrorMessage = null;

		// make sure the response came back okay
		if ( !$license_data ) {

			self::$lastLicenseKeyErrorMessage = __( 'Your license key could not be activated. Please contact support or try again.', 'ags-dashboard' );

		} else {

			if ( false === $license_data->success ) {

				switch( $license_data->error ) {

					case 'invalid_item_id':
						self::$lastLicenseKeyErrorMessage = __( 'Your license key doesn\'t seem to match the product you are trying to activate. If you think this is in error, please contact support.', 'ags-dashboard' );
						break;

					case 'expired':
						self::$lastLicenseKeyErrorMessage = __( 'Your license key couldn\'t be activated because it has expired. If you think this is in error, please contact support.', 'ags-dashboard' );
						break;

					case 'no_activations_left':
						self::$lastLicenseKeyErrorMessage = __( 'Your license key couldn\'t be activated because you\'ve reached the maximum number of site activation(s). You can increase the available site activation(s) by deactivating your license key on other sites, by removing sites from your license key via your account on our website, or by upgrading your license key. If you think this is in error, please contact support.', 'ags-dashboard' );
						break;

					default :

						self::$lastLicenseKeyErrorMessage = __( 'Your license key could not be activated. Please contact support or try again.', 'ags-dashboard' );
						break;
				}

			}

		}

		// Check if anything passed on a message constituting a failure
		if ( ! empty( self::$lastLicenseKeyErrorMessage ) ) {
			if ($fromLegacy && isset($license_data->expires_utc) ) {
				$registryData = [
				
					'key' => $key,
					'site' => $site,
					'expiration' => $license_data->expires_utc,
					
					// If we are activating from a legacy key, then the status is presumed to have been valid at some point, so save
					// that status regardless of the current status returned by the server
					'status' => 'valid',
					
					// No signature in this case. Current versions of the plugin will continue to work via the legacy license key
					// activation check. It won't matter that signature check may become mandatory in future versions if customer
					// isn't receiving updates anymore.
					'signature' => null
				];
			} else {
				return false;
			}
		} else {
			$registryData = [
				'key' => $key,
				'site' => $site,
				'expiration' => $license_data->expires_utc,
				'status' => $license_data->license,
				'signature' => $license_data->signature,
			];
			
			if (!$fromLegacy) {
				$registryData['owner'] = get_current_user_id();
			}
		}
		
		
		
		self::registerLicenseKey(
			$site,
			$itemId,
			$registryData
		);
		
		return self::$licenseKeys[$site.$itemId];
		
	}
	
	static function registerLicenseKey($site, $productId, $data) {
		$licenseKeys = get_option('aspengrove_license_keys', []);
		$licenseKeys[$site.$productId] = $data;
		update_option('aspengrove_license_keys', $licenseKeys);
		
		if ( is_multisite() ) {
			self::setMultisiteLicenseKey($site, $productId, $data);
		}
		
		self::$licenseKeys = $licenseKeys;
	}
	
	static function unregisterLicenseKey($site, $productId) {
		$licenseKeys = get_option('aspengrove_license_keys', []);
		unset( $licenseKeys[$site.$productId] );
		update_option('aspengrove_license_keys', $licenseKeys);
		self::$licenseKeys = $licenseKeys;
	}
	
	static function updateLicenseKeyRegistration($site, $productId, $data) {
		$licenseKeys = get_option('aspengrove_license_keys', []);
		$licenseKeys[$site.$productId] = array_merge($licenseKeys[$site.$productId], $data);
		update_option('aspengrove_license_keys', $licenseKeys);
		
		if ( is_multisite() ) {
			self::setMultisiteLicenseKey($site, $productId, $data);
		}
		
		self::$licenseKeys = $licenseKeys;
	}
	
	static function setMultisiteLicenseKey($site, $productId, $data) {
		$licenseKeys = get_network_option('aspengrove_license_keys', []);
		if ( !empty($data['key']) && $data['status'] == 'valid' ) {
			if ( !isset( $licenseKeys[$site.$productId] ) ) {
				$licenseKeys[$site.$productId] = [];
			}
			$licenseKeys[$site.$productId][ get_current_blog_id() ] = [
				'key' => $data['key'],
				'expiration' => $data['expiration']
			];
			update_network_option(null, 'aspengrove_license_keys', $licenseKeys);
		}
	}
	
	static function unsetMultisiteLicenseKey($site, $productId) {
		$licenseKeys = get_network_option('aspengrove_license_keys', []);
		$currentBlogId = get_current_blog_id();
		if ( isset( $licenseKeys[ $site.$productId ][ $currentBlogId ] ) ) {
			unset( $licenseKeys[ $site.$productId ][ $currentBlogId ] );
			if (!$licenseKeys[ $site.$productId ]) {
				unset($licenseKeys[ $site.$productId ]);
			}
			update_network_option(null, 'aspengrove_license_keys', $licenseKeys);
		}
	}

	static function deactivateLicenseKey($site, $itemId, $key) {

		// data to send in our API request
		$api_params = array(
			'edd_action' => 'deactivate_license',
			'license'    => $key,
			//'item_name'  => urlencode( ds_icon_expansion_ITEM_NAME ), // the name of our product in EDD
			'item_id' => $itemId,
			'url'        => home_url()
		);

		$license_data = self::requestJson(self::SITE_URL, $api_params);
		
		// $license_data->license will be either "deactivated" or "failed"
		$result = ( isset($license_data->license) && $license_data->license === 'deactivated' );
		
		self::unregisterLicenseKey($site, $itemId);

		self::$lastLicenseKeyErrorMessage = $result ? null : __('Something went wrong while deactivating your license key. The activation for this site may not have been removed from your key, and you may need to remove it manually in your account.', 'ags-dashboard');

		return $result;
	}
	
	static function onPluginActivate() {
		$adminRole = get_role('administrator');
		$adminRole->add_cap('ags_dashboard_edit_license_keys');
		
		if (!wp_next_scheduled('ags_dashboard_check_license_keys')) {
			wp_schedule_event( time() + 86400, 'daily', 'ags_dashboard_check_license_keys' );
		}
	}
	
	static function onPluginDeactivate() {
		self::$isSelfDeactivating = true;
		
		$nextLicenseKeysCheck = wp_next_scheduled('ags_dashboard_check_license_keys');
		if ($nextLicenseKeysCheck) {
			wp_unschedule_event( $nextLicenseKeysCheck, 'ags_dashboard_check_license_keys' );
		}
		
		self::$filterLegacyOptions = false;
		foreach (self::getInstalledProducts() as $product) {
			$licenseKeyInfo = self::getProductLicenseKeyInfo($product['ids'], false, true);
			if ( !empty($licenseKeyInfo['key']) && isset( $product['legacy']['key'] ) && isset( $product['legacy']['status'] ) ) {
				self::setLegacyOption($product['legacy']['key'], $licenseKeyInfo['key']);
				self::setLegacyOption($product['legacy']['status'], $licenseKeyInfo['status']);
			}
		}
	}
	
	static function changeActiveStatusAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		if ( empty( $_GET['ags_dashboard_nonce'] ) || empty( $_GET['type'] ) || empty( $_GET['name'] ) || !isset( $_GET['active'] )
				// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- nonce
				|| !wp_verify_nonce( $_GET['ags_dashboard_nonce'], 'ags_dashboard_active_status_'.((int) !empty($_GET['active'])).'_'.$_GET['name'] )
		) {
			wp_send_json_error();
		}
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized
		$itemName = $_GET['name'];
		
		switch ($_GET['type']) {
			case 'plugin':
		
				
				if (empty($_GET['active'])) {
					
					
					// wp-admin\plugins.php
					if ( ! current_user_can( 'deactivate_plugin', $itemName ) ) {
						wp_send_json_error( __( 'Sorry, you are not allowed to deactivate this plugin.', 'ags-dashboard' ) );
					}
					
					if ( is_plugin_active_for_network( $itemName ) ) {
						wp_send_json_error( __( 'Cannot deactivate a network-activated plugin.', 'ags-dashboard' ) );
					}
					
					deactivate_plugins( $itemName );
					
					$result = !is_plugin_active($itemName);
					if (!$result) {
						wp_send_json_error( __( 'The plugin was not deactivated. Please try deactivating via the Plugins page for more details on any errors that occur when this plugin is deactivated.', 'ags-dashboard' ) );
					}

					update_option( 'recently_activated', array( $itemName => time() ) + (array) get_option( 'recently_activated' ) );
					
					$linkText = __('Activate Plugin', 'ags-dashboard');
					$isActive = false; 
					
				} else {
					
					
					// wp-admin\plugins.php
					if ( ! current_user_can( 'activate_plugin', $itemName ) ) {
						wp_send_json_error( __( 'Sorry, you are not allowed to activate this plugin.', 'ags-dashboard' ) );
					}

					if ( is_network_only_plugin( $itemName ) ) {
						wp_send_json_error( __( 'Cannot activate a network-only plugin.', 'ags-dashboard' ) );
					}
					
					ob_start();
					activate_plugin( $itemName );
					$output = ob_get_clean();
					
					$result = is_plugin_active($itemName);
					if (!$result) {
						wp_send_json_error( __( 'The plugin could not be activated. Please try activating via the Plugins page for more details on any errors that occur when this plugin is activated.', 'ags-dashboard' ) );
					}
					
					if ($output) {
						$errorMessage = __( 'The plugin was activated, but it produced some unexpected output on activation. Please ensure that it is working properly on your site.', 'ags-dashboard' );
					}
					
					$linkText = __('Deactivate Plugin', 'ags-dashboard');
					$isActive = true;
				}
				break;
			
			case 'theme':
				if (empty($_GET['active'])) {
					wp_send_json_error( __( 'To deactivate this theme, please choose a different theme in Appearance > Themes.', 'ags-dashboard' ) );
				}
				
				// wp-admin\themes.php
				if ( !current_user_can( 'switch_themes' ) ) {
					wp_send_json_error( __( 'Your user account does not have permission to switch themes on this site.', 'ags-dashboard' ) );
				}
				
				$theme = wp_get_theme( $itemName );

				if ( ! $theme->exists() || ! $theme->is_allowed() ) {
					wp_send_json_error( __( 'The requested theme does not exist.', 'ags-dashboard' ) );
				}

				switch_theme( $theme->get_stylesheet() );
				
				$linkText = '';
				$isActive = true;
				
				break;
				
		}
		
		$response = [
			// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized
			'linkHref' => $linkText ? self::getChangeActiveStatusUrl( admin_url('admin-ajax.php'), $_GET['name'], $_GET['type'], $_GET['active'] ) : '',
			'linkText' => $linkText,
			'status' => $isActive ? __('Active', 'ags-dashboard') : __('Inactive', 'ags-dashboard'),
			'statusClassPrefix' => 'ags-dashboard-active',
			'statusClass' => $isActive ? 'ags-dashboard-active' : '',
			'rowClassRemovePrefix' => 'ags-dashboard-product-status-',
			'rowClassAdd' => 'ags-dashboard-product-status-'.( $isActive ? 'active' : 'inactive' )
		];
		
		if (!empty($errorMessage)) {
			$response['error'] = $errorMessage;
		}
		
		wp_send_json_success($response);
		
	}
	
	static function getChangeActiveStatusUrl($ajaxUrl, $productName, $productType, $isCurrentlyActive) {
		return add_query_arg(
			[
				'action' => 'ags_dashboard_change_active_status',
				'type' => $productType,
				'name' => $productName,
				'active' => (int) !$isCurrentlyActive,
				'ags_dashboard_nonce' => wp_create_nonce( 'ags_dashboard_active_status_'.((int) !$isCurrentlyActive).'_'.$productName )
			],
			$ajaxUrl
		);
	}
	
	static function getCurrentUserAccount() {
		if (!isset(self::$currentUserAccount)) {
			include_once(__DIR__.'/includes/account.php');
			self::$currentUserAccount = new AGSDashboardAccount('divispace');
		}
		return self::$currentUserAccount;
	}
	
	static function getCurrentUserLicenseKeys() {
		if (!isset(self::$currentUserLicenseKeys)) {
			self::$currentUserLicenseKeys = [];
			$account = self::getCurrentUserAccount();
			if ($account->isLoggedIn() ) {
				try {
					$accountSite = $account->getSiteId();
					$accountLicenseKeys = $account->getApi()->get_license_keys();
					if ( !empty( $accountLicenseKeys['success'] ) && isset( $accountLicenseKeys['data'] ) && is_array( $accountLicenseKeys['data'] ) ) {
						foreach ( $accountLicenseKeys['data'] as $licenseKey => $licenseKeyData) {
							$licenseKeyData['key'] = $licenseKey;
							$licenseKeyData['site'] = $accountSite;
							self::$currentUserLicenseKeys[] = $licenseKeyData;
						}
					}
				} catch (Exception $ex) {

				}
			}
		}
		return self::$currentUserLicenseKeys;
	}
	
	static function getCurrentUserLicenseKeysByName($name, $site='any') {
		$keys = [];
		foreach ( self::getCurrentUserLicenseKeys() as $data ) {
			if ( ( $site === 'any' || $data['site'] === $site ) && !strcasecmp($data['title'], $name) ) {
				$keys[] = $data;
			}
		}
		
		return $keys;
	}
	
	static function getCurrentUserLicenseKeysByProductIds($ids) {
		$keys = [];
		
		foreach ( self::getCurrentUserLicenseKeys() as $data ) {
			if ( isset( $ids[ $data['site'] ] ) && in_array( $ids[ $data['site'] ], $data['products'] ) ) {
				$priority = stripos($data['title'], 'membership') !== false ? 1 : 0;
				if (isset($keys[$priority])) {
					$keys[$priority][] = $data;
				} else {
					$keys[$priority] = [$data];
				}
			}
		}
		
		$keysFlat = [];
		foreach ($keys as $priorityKeys) {
			usort($priorityKeys, function($key1, $key2) {
				return strcmp($key1['title'], $key2['title']);
			});
			$keysFlat = array_merge($keysFlat, $priorityKeys);
		}
		
		return $keysFlat;
	}
	
	
	static function installAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
		
		
		if ( empty($_GET['id']) || empty($_GET['site']) || empty($_GET['type']) || empty($_GET['nonce']) || !in_array($_GET['type'], ['plugin', 'theme']) ) {
			wp_send_json_error( __( 'Missing or invalid parameters.', 'ags-dashboard' ) );
		}
		
		$isWpOrg = $_GET['site'] == 'wordpress.org';
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- no need to sanitize for nonce verification
		if ( !wp_verify_nonce($_GET['nonce'], 'ags_dashboard_install_'.$_GET['site'].$_GET['id']) ) {
			wp_send_json_error( __( 'Your request has expired; please try again.', 'ags-dashboard' ) );
		}
		
		// wp-includes\rest-api\endpoints\class-wp-rest-plugins-controller.php
		
		$skin = new WP_Ajax_Upgrader_Skin();
		
		switch ($_GET['type']) {
			case 'plugin':
				if ( !current_user_can('install_plugins') || ( !is_multisite() && !current_user_can('activate_plugins') ) ) {
					wp_send_json_error( __( 'You do not have sufficient permissions to perform this action.', 'ags-dashboard' ) );
				}
				$upgrader = new Plugin_Upgrader( $skin );
				break;
			case 'theme':
				if ( !current_user_can('install_themes') || ( !is_multisite() && !current_user_can('switch_themes') ) ) {
					wp_send_json_error( __( 'You do not have sufficient permissions to perform this action.', 'ags-dashboard' ) );
				}
				$upgrader = new Theme_Upgrader( $skin );
		}
		
		if ( $isWpOrg ) {
			
			switch ($_GET['type']) {
				case 'plugin':
					include_once ABSPATH . 'wp-admin/includes/plugin-install.php';
					
					// wp-admin\includes\ajax-actions.php
					$api = plugins_api(
						'plugin_information',
						array(
							// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized
							'slug'   => $_GET['id'],
							'fields' => array(
								'sections' => false,
							),
						)
					);
					break;
				
				case 'theme':
					// wp-admin\includes\ajax-actions.php
					include_once ABSPATH . 'wp-admin/includes/theme.php';

					$api = themes_api(
						'theme_information',
						array(
							'slug'   => $slug,
							'fields' => array( 'sections' => false ),
						)
					);
					break;
			}
			
			

			if ( empty($api->download_link) ) {
				wp_send_json_error( __( 'Unable to retrieve download URL.', 'ags-dashboard' ) );
			}
			
			$downloadUrl = $api->download_link;
			
		} else {
		
			$account = self::getCurrentUserAccount();
			
			// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- compared against array keys
			if ( !$account->isLoggedIn() ) {
				wp_send_json_error( __( 'You do not have a logged-in account for this site.', 'ags-dashboard' ) );
			}
			
			
			$originalDownloadId = (int) $_GET['id'];
			$actualDownloadId = $originalDownloadId;
			// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- site is checked against user accounts above
			$catalog = self::getProductCatalog();
			if ($catalog) {
				
				$alternatePriority = 0;
				foreach ($catalog as $productId => $product) {
					if ( isset($product['alternateOf']) && $product['alternateOf'] == $originalDownloadId
							&& isset($product['alternatePriority']) && (int) $product['alternatePriority'] > $alternatePriority
							// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- site is checked against user accounts above
							&& self::getCurrentUserLicenseKeysByProductIds([ $_GET['site'] => substr($productId, 1) ]) ) {
						
						$actualDownloadId = substr($productId, 1);
						$alternatePriority = (int) $product['alternatePriority'];
						
					}
				}
			}
			
			
			$downloadUrlRequest = [
				'downloadId' => $actualDownloadId
			];
			
			if (isset($_GET['key'])) {
				// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized
				$downloadUrlRequest['licenseKey'] = $_GET['key'];
			}
			
			// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- compared against array keys above
			$downloadUrl = $account->getApi()->get_download_url($downloadUrlRequest);
			
			if ( empty($downloadUrl['success']) || empty($downloadUrl['data']['url']) ) {
				wp_send_json_error( __( 'Unable to retrieve download URL.', 'ags-dashboard' ) );
			}
			$downloadUrl = $downloadUrl['data']['url'];
		}
		
		$result = $upgrader->install( $downloadUrl );

		if ( is_wp_error( $result ) || is_wp_error( $skin->result ) || $skin->get_errors()->has_errors() ) {
			wp_send_json_error( __( 'Installation failed.', 'ags-dashboard' ) );
		}
		
		if ( is_null( $result ) ) {
			global $wp_filesystem;
			if ( $wp_filesystem instanceof WP_Filesystem_Base && is_wp_error( $wp_filesystem->errors ) && $wp_filesystem->errors->has_errors() ) {
				wp_send_json_error( __( 'Unable to connect to the filesystem.', 'ags-dashboard' ) );
			}
			wp_send_json_error( __( 'Unable to connect to the filesystem.', 'ags-dashboard' ) );
		}

		
		switch ($_GET['type']) {
			case 'plugin':
				$item = $upgrader->plugin_info();
				break;
			case 'theme':
				$item = $upgrader->theme_info();
		}

		if ( !$item ) {
			wp_send_json_error( __( 'Something went wrong while checking the installed item.', 'ags-dashboard' ) );
		}

		add_action('shutdown', [__CLASS__, 'installAjaxResponse']);

		if ( !is_multisite() ) {
			
			// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- activation function is responsible for any sanitization needed
			if ( isset($_GET['key']) && !self::_activateLicenseKey($_GET['site'], $actualDownloadId, $_GET['key'], false) ) {
				wp_send_json_error( __( 'The item was installed but the license key could not be activated.', 'ags-dashboard' ) );
			}
			
			// Do not allow redirects (e.g. to product admin page) on product activation
			add_filter('wp_redirect', '__return_false');

			// wp-includes\rest-api\endpoints\class-wp-rest-plugins-controller.php
			switch ($_GET['type']) {
				case 'plugin':
					$activated = activate_plugin( $item );
					break;
				case 'theme':
					switch_theme( $item->stylesheet );
					$activated = get_stylesheet() == $item->stylesheet;
			}

		}

		exit; // Response is sent by installAjaxResponse() before actually exiting
	}
	
	static function installAjaxResponse() {
		self::$products = null;
		
		// phpcs:disable ET.Sniffs.ValidatedSanitizedInput.InputNotValidated, WordPress.Security.NonceVerification -- input validation and nonce checks are done in the installAjax function which is the only caller of this function (via a hook)
		ob_start();
		self::productsPage(true);
		$productsPage = ob_get_clean();
		
		$return = [
			'productsCatalog' => $productsPage,
		];
		
		$productId = sanitize_text_field($_GET['id']);
		$product = self::getInstalledProduct([ $_GET['site'] => $productId ], ['name', 'url']);
		
		if (empty($product['active']) && $_GET['site'] === 'wordpress.org') {
			$activePlugins = get_option('active_plugins');
			if (is_array($activePlugins)) {
				$pluginDir = $productId.'/';
				$pluginDirLength = strlen($pluginDir);
				
				foreach ( $activePlugins as $pluginFile ) {
					if ( substr($pluginFile, 0, $pluginDirLength) == $pluginDir ) {
						$isActiveWpOrgPlugin = true;
						break;
					}
				}
			}
		}
		
		// Note: For wordpress.org products on multisite, we assume successful install if there was no error reported by the installer
		if ( is_multisite() ? (!empty($product) || $_GET['site'] === 'wordpress.org') : (!empty($product['active']) || !empty($isActiveWpOrgPlugin)) ) {
			$return['message'] = sprintf( is_multisite() ? __('%s has been installed!', 'ags-dashboard') : __('%s has been installed and activated!', 'ags-dashboard'), empty($product['name']) ? __('The selected product', 'ags-dashboard') : $product['name'] );
			$return['messageLinks'] = [];
			
			if (!empty($product['adminPage']) && !is_multisite()) {
				$return['messageLinks'][__('View Admin Page', 'ags-dashboard')] = admin_url( $product['adminPage'] );
			}
			
			if (!empty($product['url'])) {
				$return['messageLinks'][__('View Product Page', 'ags-dashboard')] = $product['url'];
			}
			
			if (!empty($product['docs'])) {
				$return['messageLinks'][__('Documentation', 'ags-dashboard')] = $product['docs'];
			}
		} else {
			wp_send_json_error( __( 'Something went wrong while installing or activating the item.', 'ags-dashboard' ) );
		}
		
		wp_send_json_success($return);
		// phpcs:enable ET.Sniffs.ValidatedSanitizedInput.InputNotValidated WordPress.Security.NonceVerification
	}
	
	static function showAccountsTable($includeActions=true) {

		$account = self::getCurrentUserAccount();

		if ( isset($_POST['ags-dashboard-disconnect']) && isset($_POST['ags-dashboard-nonce']) ) {
			if ( wp_verify_nonce( $_POST['ags-dashboard-nonce'], 'ags-dashboard-disconnect-'.$_POST['ags-dashboard-disconnect'] ) ) {
				// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- input validated via isset() check above
				$account->logout();
			}
		}

		?>
		
		<?php if ( $account->isLoginLocked() ) { ?>
		<div class="notice notice-error ags-dashboard-notice">
			<p><?php esc_html_e('For customer security, we have temporarily locked this login email on one or more of our sites because there may have been 5 or more recent failed login attempts. Please wait 5 minutes and try again.', 'ags-dashboard'); ?></p>
		</div>
		<?php } ?>

		<table class="ags-dashboard-accounts-table">
			<thead>
				<tr>
					<th></th>
					<th><?php esc_html_e('Status', 'ags-dashboard'); ?></th>
                    <th><?php esc_html_e('Account', 'ags-dashboard'); ?></th>
					<?php if ( $includeActions ) { ?>
					<th><?php esc_html_e('Actions', 'ags-dashboard'); ?></th>
					<?php } ?>
				</tr>
			</thead>
			<tbody>
				<?php $isLoggedIn = $account->isLoggedIn(); ?>
                <?php $statusClass = ($isLoggedIn) ? "ags-dashboard-account-status-logged-in" : "ags-dashboard-account-status-not-logged-in" ;?>
				<tr class="<?php echo( esc_html( $statusClass )) ?>">
					<td>
						<img src="<?php echo( esc_url( self::$pluginBaseUrl.'images/wpzone-thumb.png' ) ); ?>"  class="ags-dashboard-account-thumb">
						<span class="ags-dashboard-account-name"><?php echo( esc_html( 'WP Zone' ) ); ?></span>
						<a href="https://wpzone.co/" target="_blank" class="ags-dashboard-site-url">wpzone.co</a>
					</td>
					<td class="ags-dashboard-account-table-status-col">
						<?php if ( $isLoggedIn ) {
						    echo file_get_contents(self::$pluginDirectory.'images/svg/check-circle.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                            esc_html_e('Logged in', 'ags-dashboard');
						} else {
							 echo file_get_contents(self::$pluginDirectory.'images/svg/times-circle.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                            esc_html_e('Not logged in', 'ags-dashboard');
						} ?>
					</td>
					<td class="ags-dashboard-account-table-account-col">
						<?php
							if ( $isLoggedIn ) {
                                echo file_get_contents(self::$pluginDirectory.'images/svg/connect-green.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
								echo(
									esc_html(
										sprintf(
											// translators: %s is email address
											__('Connected with %s', 'ags-dashboard'),
											$account->getAccountEmail()
										)
									)
								);
							} else {
                                echo file_get_contents(self::$pluginDirectory.'images/svg/connect.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
								$loginError = $account->getLastLoginError();
								echo( esc_html( empty($loginError) ? __('Not connected', 'ags-dashboard') : $loginError ) );
							}
						?>
					</td>
					<?php if ( $includeActions ) { ?>
						<td class="ags-dashboard-account-table-actions-col">
							<?php if ( $isLoggedIn ) { ?>
								<form method="post">
									<?php wp_nonce_field( 'ags-dashboard-disconnect-divispace', 'ags-dashboard-nonce', false ); ?>
									<input type="hidden" name="ags-dashboard-disconnect" value="divispace">
									<button class="ags-dashboard-btn-primary ags-dashboard-sign-out"><?php esc_html_e('Disconnect', 'ags-dashboard'); ?></button>
								</form>
							<?php } else { ?>
								<a href="<?php echo(esc_url(add_query_arg('wizard', 1))); ?>" class="ags-dashboard-btn-secondary ags-dashboard-sign-in"><?php esc_html_e('Connect', 'ags-dashboard'); ?></a>
							<?php } ?>
						</td>
					<?php } ?>
				</tr>
			</tbody>
		</table>
	<?php
	}
	
	static function getMembershipProducts() {
		$catalog = self::getProductCatalog();
		$memberships = array_filter($catalog, function($item) {
			return isset($item['type']) && $item['type'] == 'membership';
		});
		uasort($memberships, function($a, $b) {
			return strcmp($a['name'], $b['name']);
		});
		return $memberships;
	}
	
	static function getMergedCatalog($excludeInstalled=false) {
		
		if ( !isset(self::$mergedCatalog) ) {
			self::$mergedCatalog = [];

			$catalog = self::getProductCatalog();
			$siteId = 'divispace';

			$catalogProductFiles = [];

			$alternateProductIds = [];

			foreach ($catalog as $productId => $product) {

				$productId = $productId[0] == 'P' ? substr($productId, 1) : $productId;

				if ( empty($product['alternateOf']) ) {
					$existingProductIndex = empty($product['file']) ? false : array_search($product['file'], $catalogProductFiles);

					if ( $existingProductIndex === false ) {
						$product['ids'] = [ $siteId => $productId ];
						if ( !empty($product['file']) ) {
							$catalogProductFiles[ count(self::$mergedCatalog) ] = $product['file'];
						}
						self::$mergedCatalog[] = $product;
					} else {
						self::$mergedCatalog[ $existingProductIndex ]['ids'][$siteId] = $productId;
					}
				} else {
					// This is an alternate product

					if (!isset($alternateProductIds[ $product['alternateOf'] ])) {
						$alternateProductIds[ $product['alternateOf'] ] = [];
					}
					if (!isset($alternateProductIds[ $product['alternateOf'] ][ $siteId ])) {
						$alternateProductIds[ $product['alternateOf'] ][ $siteId ] = [];
					}
					$alternateProductIds[ $product['alternateOf'] ][ $siteId ][] = $productId;
				}

			}

			foreach ($alternateProductIds as $productId => $productAlternateIds) {
				foreach (self::$mergedCatalog as &$product) {
					if (isset($product['ids'][$siteId]) && $product['ids'][$siteId] == $productId) {
						if (isset($product['alternateIds'])) {
							foreach ( $productAlternateIds as $siteId => $siteProductIds ) {
								if (isset($product['alternateIds'][$siteId])) {
									$product['alternateIds'][$siteId] = array_merge( $product['alternateIds'][$siteId], $siteProductIds );
								} else {
									$product['alternateIds'][$siteId] = $siteProductIds;
								}
							}
						} else {
							$product['alternateIds'] = $productAlternateIds;
						}
					}
				}
			}

		}

		
		if ( $excludeInstalled ) {
			$filteredCatalog = [];
			
			$installedProducts = self::getInstalledProducts();
			
			foreach ( self::$mergedCatalog as $product ) {
				foreach ( $installedProducts as $installedProduct ) {
					if ( array_intersect_assoc( $installedProduct['ids'], $product['ids']) ) {
						continue 2;
					}
					
					if ( !empty( $product['alternateIds'] ) ) {
						foreach ( $product['alternateIds'] as $siteId => $alternateIds ) {
							foreach ($alternateIds as $alternateId) {
								if ( array_intersect_assoc( $installedProduct['ids'], [ $siteId => $alternateId ] ) ) {
									continue 4;
								}
							}
						}
					}
					
				}
				$filteredCatalog[] = $product;
			}
			
			return $filteredCatalog;
			
		}
		
		return self::$mergedCatalog;
	}
	
	static function getInstalledProduct($productIds, $extraFields=[]) {
		foreach (self::getInstalledProducts($extraFields) as $product) {
			if ( array_intersect_assoc( $productIds, $product['ids']) ) {
				return $product;
			}
		}
	}
	
	static function getInstalledProductTitle($productIds) {
		
		foreach (self::getInstalledProducts(['name']) as $product) {
			if ( array_intersect_assoc( $productIds, $product['ids']) ) {
				return isset( $product['name'] ) ? $product['name'] : '';
			}
		}
		return '';
	}
	
	static function getSupportArticles($searchQuery=null) {
		
		if (!$searchQuery) {
			$cached = get_transient('ags_dashboard_support_articles');
			
			if ($cached) {
				return @json_decode($cached);
			}
			
			$installedProducts = self::getInstalledProducts(['name']);
		}
		
		
		$articles = self::getCurrentUserAccount()->getApi()->query_help_articles(
			[
				'request' => $searchQuery
					? [
						'type' => 'search',
						'query' => $searchQuery,
						'limit' => 50
					]
					: [
						'type' => 'list',
						'products' => array_column($installedProducts, 'name' ),
						'productTypes' => array_unique( array_column( $installedProducts, 'type' ) ),
						'limit' => 20
					]
			]
		);
		
		if ( empty($articles['success']) ) {
			throw new Exception();
		}
		
		if (!$searchQuery) {
			set_transient( 'ags_dashboard_support_articles', wp_json_encode($articles['data']), 86400 );
		}
		
		
		return $articles['data'];
	}
	
	static function getSupportArticlesAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		try {
			// phpcs:ignore WordPress.Security.NonceVerification -- read-only
			$articles = self::getSupportArticles( isset($_POST['q']) ? sanitize_text_field($_POST['q']) : null );
		} catch (Exception $ex) {
			wp_send_json_error();
		}
		
		wp_send_json_success($articles);
	}
	
	static function checkUpdatesAjax() {
		if ( !current_user_can('manage_options') || (is_multisite() && !is_super_admin()) ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- nonce
		if ( empty($_GET['nonce']) || !wp_verify_nonce($_GET['nonce'], 'ags_dashboard_check_updates') ) {
			wp_send_json_error( __('Please refresh the page and try again', 'ags-dashboard') );
		}
		
		// Remove cached update data
		delete_site_transient( 'aspengrove_updates' );
		delete_site_transient( 'update_plugins' );
		delete_site_transient( 'update_themes' );
		
		wp_send_json_success( [ 'redirect' => admin_url('update-core.php') ] );
		
	}
	
	static function refreshCatalogAjax() {
		if ( !current_user_can('manage_options') || (is_multisite() && !is_super_admin()) ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- nonce
		if ( empty($_GET['nonce']) || !wp_verify_nonce($_GET['nonce'], 'ags_dashboard_refresh_catalog') ) {
			wp_send_json_error( __('Please refresh the page and try again', 'ags-dashboard') );
		}
		
		// Remove cached catalog
		if (is_network_admin()) {
			delete_site_transient( 'divispace_product_catalog' );
		} else {
			delete_transient( 'divispace_product_catalog' );
		}
		
		wp_send_json_success( [ 'redirect' => admin_url('admin.php?page=ags-dashboard-products') ] );
		
	}
	
	static function generateLegacySupportPlugin() {
		
		if ( !empty( self::$isSelfDeactivating ) ) {
			@unlink(ABSPATH.'/wp-content/mu-plugins/ags-dashboard-legacy-support.php');
			return;
		}
		
		$legacyData = self::getProductCatalog();
		if ( !is_array($legacyData) ) {
			return false;
		}
		
		$updaterClasses = [];
		$keyOptions = [];
		$statusOptions = [];
		$redirectPages = [];
		$css = '';
		$hasActiveTheme = false;
		self::$productsWithUpdaters = [];
		
		// Bypass cache
		self::$products = null;
		
		foreach ( self::getInstalledProducts() as $product ) {
			
			if ( isset($product['ids']['divispace']) ) {
				$productLegacyData = [];
				$productId = 'P'.$product['ids']['divispace'];
				
				foreach (['updaterClass', 'helperHideSelector', 'helperSupportSince'] as $legacyField) {
					if ( isset( $product[$legacyField] ) ) {
						$productLegacyData[$legacyField] = $product[$legacyField];
					} else if ( isset( $legacyData[ $productId ][$legacyField] ) ) {
						$productLegacyData[$legacyField] = $legacyData[ $productId ][$legacyField];
					}
				}
				
				if ($product['type'] == 'theme' ) {
					self::$productsWithUpdaters[] = basename(dirname($product['file'])).'/'.basename($product['file']);
				} else if ( isset( $productLegacyData['updaterClass'] ) ) {
					self::$productsWithUpdaters[] = plugin_basename($product['file']);
				}
				
				if ( !empty($product['active']) ) {
					
					if ($product['type'] == 'theme') {
						$hasActiveTheme = $product['ids'];
					}
				
					
					if ( isset( $productLegacyData['updaterClass'] ) ) {
						$updaterClasses[ plugin_basename($product['file']) ] = $productLegacyData['updaterClass'];
					}
					
					if ( isset( $productLegacyData['helperHideSelector'] ) ) {
						$css .= $productLegacyData['helperHideSelector'].'{display:none;}';
					}
					
					if (
						(
							empty( $productLegacyData['helperSupportSince'] )
							|| version_compare( self::getProductVersion($product['file']), $productLegacyData['helperSupportSince'], '<' )
						)
						&& isset( $product['legacy']['key'] ) && isset( $product['legacy']['status'] )
					) {
						
						$keyOptions[ $product['legacy']['key'] ] = $product['ids'];
						$statusOptions[ $product['legacy']['status'] ] = $product['ids'];
						
						if ( isset( $product['adminPage'] ) ) {
							
							$queryStartPos = strpos($product['adminPage'], '?');
							
						
							
							if ($queryStartPos) {
								$query = [];
								parse_str( substr($product['adminPage'], $queryStartPos + 1), $query );
								$redirectPages[] = [
									'page' => substr($product['adminPage'], 0, $queryStartPos),
									'query' => $query,
									'ids' => $product['ids']
								];
							} else {
								$redirectPages[] = [
									'page' => $product['adminPage'],
									'ids' => $product['ids']
								];
							}
							
						}
					}
						
					
				}
				
			}
		}
		
		$currentPath = dirname(__FILE__);
		$dashboardPhpPath = basename(__FILE__);
		$i = 0;
		while (basename($currentPath) != 'wp-content' && $i < 100) {
			$dashboardPhpPath = basename($currentPath).DIRECTORY_SEPARATOR.$dashboardPhpPath;
			$currentPath = dirname($currentPath);
			++$i;
		}
		
		if ($i == 100) {
			$dashboardPhpPath = var_export(__FILE__, true);
		} else {
			$dashboardPhpPath = 'ABSPATH.'.var_export(DIRECTORY_SEPARATOR.'wp-content'.DIRECTORY_SEPARATOR.$dashboardPhpPath, true);
		}
		
		$pluginContents = '<?php
/*
'.chr(80).'lugin Name: '.self::$pluginName.' Legacy Support
Description: This auto-generated plugin provides transitional and legacy support for WP Zone (including Aspen Grove Studios) plugins that are now using the '.self::$pluginName.' for license key and updates management.
*/

/*
This plugin was automatically generated by the '.self::$pluginName.' plugin, and is considered part of '.self::$pluginName.' for licensing and copyright purposes. For license and copyright information, please see the '.self::$pluginName.' plugin.
*/

add_action(\'plugins_loaded\', function() {
	if (class_exists(\'AGSDashboard\')) {
		AGSDashboard::$productsWithUpdaters = '.var_export(self::$productsWithUpdaters, true).';
	}
});
';

		if ($hasActiveTheme) {
			$pluginContents .= '
			class AGSDashboardThemeUpdaterPlaceholder {
				function has_license_key() {
					if (!class_exists(\'AGSDashboard\')) return false;
					$licenseKeyInfo = AGSDashboard::getProductLicenseKeyInfo(AGSDashboard::getActiveThemeProductIds());
					return isset($licenseKeyInfo[\'status\']) && ($licenseKeyInfo[\'status\'] === \'valid\' || $licenseKeyInfo[\'status\'] === \'expired\');
				}
				function license_key_box() {}
				function activate_page() {
					echo(\'<script>location.href="\'.str_replace(\'"\', \'\\\\"\', admin_url(\'admin.php?page=ags-dashboard-products&needsActivation=\'.rawurlencode(AGSDashboard::getInstalledProductTitle('.var_export($hasActiveTheme, true).')))).\'";</script>\');
				}
			}
			
			add_action(\'after_setup_theme\', function() {
									if ( remove_action(\'after_setup_theme\', \'AGS_THEME_updater\') ) {
										global $AGS_THEME_updater;
										$AGS_THEME_updater = new AGSDashboardThemeUpdaterPlaceholder();
									}
									$tgmPaths = [
										get_stylesheet_directory().\'/admin/aspen-plugin-installer/class-tgm-plugin-activation.php\',
										get_stylesheet_directory().\'/aspen-plugin-installer/class-tgm-plugin-activation.php\'
									];
									foreach ($tgmPaths as $tgmPath) {
										if (file_exists($tgmPath)) {
											@include_once($tgmPath);
											break;
										}
									}
								}, 1);';
		}
		
		foreach ($updaterClasses as $productFile => $className) {
			$pluginContents .= 'class '.$className.' {}'."\n";
		}
		
		foreach ($keyOptions as $optionName => $productIds) {
			
			if (substr($optionName, 0, 3) == 'et.') {
				$optionName = substr($optionName, 3);
				$filters = [ 'et_get_option_et_divi_'.$optionName, 'et_get_option_et_extra_'.$optionName ];
			} else {
				$filters = [ 'pre_option_'.$optionName ];
			}
			
			foreach ($filters as $filter) {
				$pluginContents .= 'add_filter( '.var_export($filter, true).', '
									.'function($value) { require_once('.$dashboardPhpPath.'); if (!AGSDashboard::$filterLegacyOptions) { return $value; } $info = AGSDashboard::getProductLicenseKeyInfo('.var_export($productIds, true).'); return $info[\'key\']; } );'."\n";
			}
		}
		
		foreach ($statusOptions as $optionName => $productIds) {
			
			if (substr($optionName, 0, 3) == 'et.') {
				$optionName = substr($optionName, 3);
				$filters = [ 'et_get_option_et_divi_'.$optionName, 'et_get_option_et_extra_'.$optionName ];
			} else {
				$filters = [ 'pre_option_'.$optionName ];
			}
			
			foreach ($filters as $filter) {
				$pluginContents .= 'add_filter( '.var_export($filter, true).', '
									.'function($value) { require_once('.$dashboardPhpPath.');  if (!AGSDashboard::$filterLegacyOptions) { return $value; } $info = AGSDashboard::getProductLicenseKeyInfo('.var_export($productIds, true).'); return $info[\'status\']; } );'."\n";
			}
		}
		
		if ($redirectPages) {
			
			$pluginContents .= 'add_action(\'admin_init\', function() { if (!class_exists(\'AGSDashboard\')) return; global $pagenow;';
		
			foreach ($redirectPages as $pageData) {
				if (!empty($pageData['query'])) {
					$pluginContents .= '$q = '.var_export($pageData['query'], true).';';
				}
				$pluginContents .= 'if ($pagenow == '.var_export($pageData['page'], true).(empty($pageData['query']) ? '' : '&& !array_diff_assoc($q, array_intersect_key($_GET, $q))').') {'
									.'$info = AGSDashboard::getProductLicenseKeyInfo('.var_export($pageData['ids'], true).');'
									.'if ($info[\'status\'] != \'valid\') wp_safe_redirect( admin_url(\'admin.php?page=ags-dashboard-products&needsActivation=\'.rawurlencode(AGSDashboard::getInstalledProductTitle('.var_export($pageData['ids'], true).'))) );'
									.'}';
			}
			
			$pluginContents .= '});';
			
		}
		
		if ($css) {
			$pluginContents .= 'add_action(\'admin_print_scripts\', function() { echo '.var_export('<style type="text/css">'.$css.'</style>', true).'; });';
		}
		
		if ( !is_dir(ABSPATH.'/wp-content/mu-plugins') ) {
			mkdir(ABSPATH.'/wp-content/mu-plugins');
		}
		
		file_put_contents(ABSPATH.'/wp-content/mu-plugins/ags-dashboard-legacy-support.php', $pluginContents);
		
	}
	
	static function maybeHandleUpdateDownload($defaultReturn, $url, $upgrader) {
		if ( !strcasecmp( substr( $url, 0, strlen(self::SITE_URL) ), self::SITE_URL ) ) {
			return self::handleUpdateDownload('divispace', $url, $upgrader);
		}
		return $defaultReturn;
	}
	
	static function handleUpdateDownload($site, $url, $upgrader) {
		
		// Based on wp-admin/includes/class-wp-upgrader.php

		$upgrader->skin->feedback( 'downloading_package', wp_parse_url(self::SITE_URL, PHP_URL_HOST) );

		add_filter( 'download_url_error_max_body_size', [__CLASS__, 'eightKb'] );
		$download_file = download_url( $url, 300 );
		remove_filter( 'download_url_error_max_body_size', [__CLASS__, 'eightKb'] );

		if ( is_wp_error( $download_file ) && ! $download_file->get_error_data( 'softfail-filename' ) ) {
			
			$errorData = $download_file->get_error_data();
			if (isset($errorData['body'])) {
				$errorResponse = strip_tags($errorData['body']);
				
				if ( strpos($errorResponse, 'expired') !== false ) {
					
					$packageInfo = explode(':', @base64_decode( basename( $url ) ) );
					
					if (isset($packageInfo[1]) && isset($packageInfo[2]) && is_numeric($packageInfo[2])) {
						$renewalUrl = sprintf(
							'%scheckout/?edd_license_key=%s&download_id=%d',
							self::SITE_URL,
							$packageInfo[1],
							$packageInfo[2]
						);
					} else {
						$renewalUrl = admin_url('admin.php?page=ags-dashboard-products');
					}
					
					return new WP_Error(
						'download_failed',
						sprintf(
							// Translators: %s are <a></a> tags
							esc_html__( 'Your license key has expired. Please %sclick here%s to renew it before updating.', 'ags-dashboard'),
							'<a href="'.esc_url($renewalUrl).'" target="_blank">',
							'</a>'
						)
					);
				} else if ( strpos($errorResponse, 'not been activated') !== false ) {
					return new WP_Error(
						'download_failed',
						sprintf(
							// Translators: %s is a link to the dashboard
							esc_html__( 'Your license key does not appear to be active for this site. This can happen if you haven\'t activated your license key yet, or if you change your site URL after activating the license key. You can manage your license key activation in %s. If your site URL has changed, please locate the license key in %1s, click More > Manage Activations, and remove any outdated site URLs before deactivating and reactivating the license key.', 'ags-dashboard'),
							'<a href="'.esc_url( admin_url('admin.php?page=ags-dashboard-products') ).'" target="_parent">'.esc_html( self::$pluginName ).'</a>',
							'<a href="'.esc_url( admin_url('admin.php?page=ags-dashboard-products') ).'" target="_parent">'.esc_html( self::$pluginName ).'</a>'
						)
					);
				}
			}
			
			return new WP_Error(
				'download_failed',
				sprintf(
							// Translators: %s are <a></a> tags
							esc_html__( 'Something went wrong while downloading your update. Please try again or %scontact support%s.', 'ags-dashboard'),
							'<a href="'.esc_url( admin_url('admin.php?page=ags-dashboard-support') ).'" target="_parent">',
							'</a>'
				)
			);
			
		}

		return $download_file;
		
	}
	
	static function eightKb() {
		return KB_IN_BYTES * 8;
	}
	
	static function checkLicenseKeys() {
		
		$licenseKeyInfoBySite = [];
		foreach ( self::getInstalledProducts() as $product ) {
			$licenseKeyInfo = self::getProductLicenseKeyInfo( $product['ids'], false );
			if ( !empty($licenseKeyInfo['key']) ) {
				if ( !isset( $licenseKeyInfoBySite[ $licenseKeyInfo['site'] ] ) ) {
					$licenseKeyInfoBySite[ $licenseKeyInfo['site'] ] = [];
				}
				$licenseKeyInfoBySite[ $licenseKeyInfo['site'] ][ $product['ids'][ $licenseKeyInfo['site'] ] ] = [
					'licenseKey' => $licenseKeyInfo['key'],
					'expiration' => $licenseKeyInfo['expiration'],
					'signature' => $licenseKeyInfo['signature']
				];
			}
		}

		$account = self::getCurrentUserAccount();
		$siteId = 'divispace';
		if ( !empty( $licenseKeyInfoBySite[$siteId] ) && $account->isLoggedIn() ) {

			try {

				$response = $account->getApi()->check_license_keys([
					'request' => [
						'siteUrl' => home_url(),
						'licenseKeys' => $licenseKeyInfoBySite[$siteId]
					]
				]);

				if ( empty($response['success']) || empty($response['data']) ) {
					throw new Exception();
				}

				foreach ($response['data'] as $productId => $result) {
					if ( empty($result['success']) ) {
						self::updateLicenseKeyRegistration( $siteId, $productId, [
							'checkError' => empty($result['error']) ? __('Unknown', 'ags-dashboard') : $result['error']
						] );
					} else if ( isset($result['signature']) ) {
						self::updateLicenseKeyRegistration( $siteId, $productId, [
							'expiration' => $result['expiration'],
							'signature' => $result['signature'],
							'checkError' => null
						] );
					} else {
						self::updateLicenseKeyRegistration( $siteId, $productId, [
							'checkError' => null
						] );
					}

				}
			} catch (Exception $ex) {
				return false;
			}
		}
		
		return true;
	}
	
	static function checkLicenseKeysAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- nonce
		if ( empty($_GET['nonce']) || !wp_verify_nonce($_GET['nonce'], 'ags_dashboard_check_license_keys') ) {
			wp_send_json_error( __('Please refresh the page and try again', 'ags-dashboard') );
		}
		
		self::checkLicenseKeys() ? wp_send_json_success(['reload' => true]) : wp_send_json_error();
	}
	
	static function showAdminNavbar($activePage) {
		$account = self::getCurrentUserAccount();
		if ($account->isLoggedIn()) {
			$loggedInId = $account->getAccountEmail();
		}
		
?>
		<div id="ags-dashboard-admin-navbar">
            <div class="ags-dashboard-logo-wrapper">
			    <img class="ags-dashboard-logo" src="<?php echo( esc_url( self::$pluginBaseUrl.'images/dashboard-logo.png') ); ?>" alt="<?php echo( esc_html(self::$pluginName) ); ?>">
            </div>
			<ul id="ags-dashboard-admin-navbar-menu">
				<li<?php if ($activePage == 'products') echo(' class="ags-dashboard-active"'); ?>><a href="?page=ags-dashboard-products"><?php esc_html_e('Products', 'ags-dashboard'); ?></a></li>
				<?php if (!is_network_admin()) { ?>
					<li<?php if ($activePage == 'support') echo(' class="ags-dashboard-active"'); ?>><a href="?page=ags-dashboard-support"><?php esc_html_e('Support', 'ags-dashboard'); ?></a></li>
				<?php } ?>
				<li<?php if ($activePage == 'settings') echo(' class="ags-dashboard-active"'); ?>><a href="?page=ags-dashboard-settings"><?php esc_html_e('Settings', 'ags-dashboard'); ?></a></li>
			</ul>
			<div id="ags-dashboard-admin-navbar-accounts">
				<?php if (empty($loggedInId)) { ?>
                    <a href="?page=ags-dashboard-settings" class="ags-dashboard-not-logged-in"><?php esc_html_e('Login', 'ags-dashboard'); ?></a>
				<?php } else { ?>
                    <div class="ags-dashboard-dropdown ags-dashboard-dropdown-dark">

                        <?php if (isset($loggedInId)) { ?>
                            <span class="ags-dashboard-account-divispace">
                                <?php echo( esc_html( $loggedInId ) ); ?>
                            </span>
                        <?php } ?>

                        <ul id="ags-dashboard-admin-navbar-accounts-dropdown">
                            <li><a href="?page=ags-dashboard-settings"><?php esc_html_e('Manage Accounts', 'ags-dashboard'); ?></a></li>
                        </ul>
                    </div>
				<?php } ?>

		    </div>
		</div>
<?php
	}
	
	static function adminNotices() {
		
		
		$queuedNotices = [];
		$userId = get_current_user_id();
		$locale = get_locale();
		
		if (current_user_can('manage_options')) {
			$now = time();
			if (!isset(self::$licenseKeys)) {
				self::$licenseKeys = get_option('aspengrove_license_keys', []);
			}
			foreach ( self::$licenseKeys as $productId => $licenseKeyInfo ) {
				if ( !empty($licenseKeyInfo['expiration']) && $licenseKeyInfo['expiration'] != 'lifetime' ) {
					if ( $licenseKeyInfo['expiration'] < $now ) {
						$queuedNotices[ 'expired_'.$productId.'_'.$licenseKeyInfo['expiration'] ] = [];
					} else if ( $licenseKeyInfo['expiration'] < $now + (86400 * 14) ) {
						$queuedNotices[ 'expiring_'.$productId.'_'.$licenseKeyInfo['expiration'] ] = [];
					}
				}
			}
		}
		
		$notices = get_user_meta($userId, 'ags_dashboard_notices', true);
		
		if ($notices) {
			$newNotices = array_merge( $queuedNotices, array_intersect_key($notices, $queuedNotices) );
			if ($newNotices != $notices) {
				$notices = $newNotices;
				$noticesChanged = true;
			}
		} else {
			$notices = $queuedNotices;
			if ($notices) {
				$noticesChanged = true;
			}
		}
		
		foreach ($notices as $noticeId => &$notice) {
			
			if (!empty($notice['dismissed'])) {
				continue;
			}
			
			if ( empty($notice['locale']) || $notice['locale'] != $locale ) {
				$notice = [
					'html' => self::getAdminNoticeHtml($noticeId),
					'locale' => $locale
				];
				$noticesChanged = true;
			}
			$adminNoticeDisplayed = true;
			
?>
			<div id="ags-dashboard-notice-<?php echo(esc_attr( $noticeId )); ?>" class="ags-dashboard-notice notice notice-warning is-dismissible"
					data-ags-dashboard-dismiss-nonce="<?php echo(esc_attr(wp_create_nonce('ags_dashboard_dismiss_'.$noticeId))); ?>">
				<p><?php echo($notice['html']); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped -- HTML output ?></p>
			</div>
<?php
			
		}

		if ( !empty( $noticesChanged ) ) {
			update_user_meta($userId, 'ags_dashboard_notices', $notices);
		}
		
		if (!empty($adminNoticeDisplayed)) {
?>
	<script>
		jQuery(document).ready(function($) {
			$('.ags-dashboard-notice').on('click', '.notice-dismiss', function() {
				var $notice = $(this).closest('.ags-dashboard-notice');
				$.post(
					ajaxurl,
					{
						action: 'ags_dashboard_admin_notice_dismiss',
						noticeId: $notice.attr('id').substr(21),
						nonce: $notice.data('ags-dashboard-dismiss-nonce')
					}
				).fail(function() {
					alert('Unable to close the notice; please refresh the page and try again.' );
				});
			});
		});
	</script>
<?php
		}
		
	}

	static function dismissAdminNoticeAjax() {
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- nonce
		if (isset($_POST['nonce']) && !empty($_POST['noticeId']) && wp_verify_nonce($_POST['nonce'], 'ags_dashboard_dismiss_'.$_POST['noticeId'])) {
			$userId = get_current_user_id();
			$notices = get_user_meta($userId, 'ags_dashboard_notices', true);
			// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- checked against known array indexes
			if (isset($notices[ $_POST['noticeId'] ])) {
				// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- checked against known array indexes above
				$notices[ $_POST['noticeId'] ] = ['dismissed' => true]; // no need to keep the other data in the notice
				update_user_meta($userId, 'ags_dashboard_notices', $notices);
				wp_send_json_success();
			}
		}
		wp_send_json_error();
	}
	
	static function getLicenseKeyRenewalUrl($siteId, $licenseKey) {
		return sprintf(
			'%scheckout/?edd_license_key=%s',
			self::SITE_URL,
			$licenseKey
		);
	}
	
	static function getAddToCartUrl($productId) {
		return sprintf(
			'%scheckout/?edd_action=add_to_cart&download_id=%d',
			self::SITE_URL,
			$productId
		);
	}
	
	static function getAdminNoticeHtml($noticeId) {
		
		$noticeParams = explode('_', $noticeId);
		
		switch ($noticeParams[0]) {
			case 'expired':
				if (count($noticeParams) == 3) {
					if ( substr( $noticeParams[1], 0, strlen('divispace') ) == 'divispace' ) {
						$ids = [ 'divispace' => substr( $noticeParams[1], strlen('divispace') ) ];
						$title = self::getInstalledProductTitle($ids);
						$licenseKeyInfo = self::getProductLicenseKeyInfo($ids);
						break;
					}
				}
				
				return
				(
					empty($title)
						? esc_html__('The license key for one of the products on this site has expired.')
						: sprintf(
							// Translators: %s is the product name
							esc_html__('The license key for %s has expired.'),
							esc_html($title)
						)
				)
				.' '
				.(
					empty($licenseKeyInfo['key'])
						? sprintf(
							// Translators: %s is a link to the products admin page
							esc_html__('Please renew the license key to continue receiving updates and support. For more information, please open %s.'),
							'<a href="'.esc_url( admin_url( 'admin.php?page=ags-dashboard-products' ) ).'">'.esc_html(self::$pluginName).'</a>'
						)
						: sprintf(
							// Translators: %s are the license key renewal link <a></a> tags
							esc_html__('To continue receiving updates and support, please %srenew the license key here%s.'),
							'<a href="'.esc_url( self::getLicenseKeyRenewalUrl($licenseKeyInfo['site'], $licenseKeyInfo['key']) ).'" target="_blank">',
							'</a>'
						)
				);
				
				break;
				
			case 'expiring':
				if (count($noticeParams) == 3) {
					if ( substr( $noticeParams[1], 0, strlen('divispace') ) == 'divispace' ) {
						$ids = [ 'divispace' => substr( $noticeParams[1], strlen('divispace') ) ];
						$title = self::getInstalledProductTitle($ids);
						$licenseKeyInfo = self::getProductLicenseKeyInfo($ids);
						break;
					}
				}
				
				return
				(
					empty($title)
						? esc_html__('The license key for one of the products on this site will expire soon.')
						: sprintf(
							// Translators: %s is the product name
							esc_html__('The license key for %s will expire soon.'),
							esc_html($title)
						)
				)
				.' '
				.(
					empty($licenseKeyInfo['key'])
						? sprintf(
							// Translators: %s is a link to the products admin page
							esc_html__('Please renew the license key to continue receiving updates and support. For more information, please open %s.'),
							'<a href="'.esc_url( admin_url( 'admin.php?page=ags-dashboard-products' ) ).'">'.esc_html(self::$pluginName).'</a>'
						)
						: sprintf(
							// Translators: %s are the license key renewal link <a></a> tags
							esc_html__('To continue receiving updates and support, please %srenew the license key here%s.'),
							'<a href="'.esc_url( self::getLicenseKeyRenewalUrl($licenseKeyInfo['site'], $licenseKeyInfo['key']) ).'" target="_blank">',
							'</a>'
						)
				);
				
				break;
		}
		
		return '';
	}
	
	static function getRemoteAccessStatus() {
		$remoteUser = get_user_by('email', 'support@aspengrovestudios.com');
		if ( is_multisite() && $remoteUser ) {
			$userSites = get_blogs_of_user($remoteUser->ID, true);
			if ( !isset($userSites[ get_current_blog_id() ]) ) {
				$remoteUser = null;
			}
		}
		
		if ($remoteUser) {
			
			
			$remoteExpiry = get_option( 'ags_dashboard_remote_access_expiry', 0 );
			$remoteEnabled = $remoteExpiry > time();
		} else {
			$remoteEnabled = false;
		}
		
		return [
			'status' => $remoteEnabled,
			'statusText' => $remoteEnabled
								? sprintf(
									__('Support access is enabled until %s. The following support ticket email address is associated with remote access: %s', 'ags-dashboard'),
									wp_date( get_option('date_format').' '.get_option('time_format'), $remoteExpiry ),
									get_option( 'ags_dashboard_remote_access_email', __('unknown', 'ags-dashboard') )
								)
								: __('Support access is disabled', 'ags-dashboard')
		];
	}
	
	static function setRemoteAccessAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- nonce
		if ( empty($_GET['nonce']) || !wp_verify_nonce($_GET['nonce'], 'ags_dashboard_set_remote_access') ) {
			wp_send_json_error( __('Unable to validate your request. Please refresh the page and try again.', 'ags-dashboard') );
		}
		
		try {
			if ( empty( $_POST['remoteAccessActive'] ) ) {
				self::setRemoteAccess( false );
			} else if ( empty( $_POST['remoteAccessEmail'] ) ) {
				throw new Exception( __('Please specify an email address', 'ags-dashboard') );
			} else {
				// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- checked with is_email in function
				self::setRemoteAccess( true, $_POST['remoteAccessEmail'] );
			}
		} catch (Exception $ex) {
			wp_send_json_error( $ex->getMessage() );
		}
		
		wp_send_json_success( self::getRemoteAccessStatus() );
	}

	static function setRemoteAccess($isActive, $email=null) {
		
		if (!current_user_can('create_users')) {
			throw new Exception( __('The create_users capability is required for this action.', 'ags-dashboard') );
		}
		
		if ( $isActive ) {
			
			if ( !is_email( $email ) ) {
				throw new Exception( __('Please specify a valid email address.', 'ags-dashboard') );
			}
		
			$remoteUser = get_user_by('email', 'support@aspengrovestudios.com');
			if (!$remoteUser) {
				
				wp_insert_user([
					'user_login' => 'ags_support',
					'user_pass' => wp_generate_password(32),
					'user_email' =>  'support@aspengrovestudios.com',
					'display_name' => self::$pluginName.' Support',
					'description' => '',
					'role' => 'administrator'
				]);
				$remoteUser = get_user_by('email', 'support@aspengrovestudios.com');
				
				if (!$remoteUser) {
					throw new Exception();
				}
				
			} else if ( is_multisite() ) { // user exists and this is multisite
				
				if ( !add_user_to_blog( get_current_blog_id(), $remoteUser->ID ) ) {
					throw new Exception();
				}
				
			}
			
			$expiry = time() + (86400 * 7);
			$token = sha1( wp_generate_password(32) );
			
			update_option( 'ags_dashboard_remote_access_token', $token, false );
			update_option( 'ags_dashboard_remote_access_expiry', $expiry, false );
			update_option( 'ags_dashboard_remote_access_email', $email, false );
			
			wp_clear_scheduled_hook('ags_dashboard_delete_remote_user');
			wp_schedule_single_event($expiry, 'ags_dashboard_delete_remote_user');
			
			$account = self::getCurrentUserAccount();
			
			$response = $account->getApi()->register_remote_access_token([
				'url' => admin_url(),
				'email' => $email,
				'token' => $token,
				'expiry' => $expiry
			]);
			
			if (empty($response['success'])) {
				throw new Exception();
			}
			
		} else {
			
			$remoteUser = get_user_by('email', 'support@aspengrovestudios.com');
			
			$account = self::getCurrentUserAccount();
			$account->getApi()->delete_remote_access_token([
				'url' => admin_url(),
				'token' => get_option( 'ags_dashboard_remote_access_token' )
			]);
			
			if ( !self::deleteRemoteAccessUser() ) {
				throw new Exception( __('Could not disable support access; maybe it was already disabled? Please refresh the page for updated status information.', 'ags-dashboard') );
			}
			
			wp_clear_scheduled_hook('ags_dashboard_delete_remote_user');
		}
		
	}
	
	static function deleteRemoteAccessUser() {
		delete_option( 'ags_dashboard_remote_access_token' );
		delete_option( 'ags_dashboard_remote_access_expiry' );
		delete_option( 'ags_dashboard_remote_access_email' );
		
		$remoteUser = get_user_by('email', 'support@aspengrovestudios.com');
		
		if ($remoteUser) {
			
			$deleteStatus = wp_delete_user($remoteUser->ID);
			
			if ( $deleteStatus && is_multisite() ) {
				if ( !get_blogs_of_user($remoteUser->ID, true) ) {
					$deleteStatus = wpmu_delete_user($remoteUser->ID);
				}
			}
			
			if ($deleteStatus) {
				return true;
			}
		
		}
		
		return false;
	}
	
	static function handleSupportAccess() {
		// phpcs:disable WordPress.Security.NonceVerification
		$failTransient = 'ags_dashboard_support_access_fail_'.( empty($_SERVER['REMOTE_ADDR']) ? '' : sanitize_text_field($_SERVER['REMOTE_ADDR']) );
		$failTransientValue = (int) get_transient($failTransient);
		if ($failTransientValue < 3) {
			
			// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotValidated -- checked before function is hooked
			$accessToken = sanitize_key( $_POST['ags_support_access'] );
			
			$remoteUser = get_user_by('email', 'support@aspengrovestudios.com');
			
			if ( is_multisite() && $remoteUser ) {
				$userSites = get_blogs_of_user($remoteUser->ID, true);
				if ( !isset($userSites[ get_current_blog_id() ]) ) {
					$remoteUser = null;
				}
			}
			
			if ($remoteUser) {
				$expiry = (int) get_option( 'ags_dashboard_remote_access_expiry', 0 );
				$token = get_option( 'ags_dashboard_remote_access_token' );
				if (
					$expiry
					&& $token
					&& $expiry > time() 
					&& (
						$accessToken == sha1( floor(time() / 3600) . $token )
						|| $accessToken == sha1( floor(time() / 3600) - 1 . $token )
					)
				) {
						
					// wp-includes\user.php
					wp_set_auth_cookie( $remoteUser->ID, false, is_ssl() );
					wp_safe_redirect(admin_url());
					exit;
					
				}
			}
			
		}
		
		set_transient( $failTransient, ++$failTransientValue, 300 );
		
		// phpcs:enable WordPress.Security.NonceVerification
	}
	
	static function openSupportTicketAjax() {
		if ( !current_user_can('manage_options') ) {
			wp_send_json_error( __('You do not have permission to access this functionality.', 'ags-dashboard') );
		}
		
		// phpcs:ignore ET.Sniffs.ValidatedSanitizedInput.InputNotSanitized -- nonce
		if ( empty($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'ags_dashboard_open_support_ticket') ) {
			wp_send_json_error( __('Unable to validate your request. Please refresh the page and try again.', 'ags-dashboard') );
		}
		
		if ( empty($_POST['ticket']['name']) || empty($_POST['ticket']['email']) || empty($_POST['ticket']['product']) || empty($_POST['ticket']['subject']) || empty($_POST['ticket']['message']) ) {
			wp_send_json_error( __('One or more fields were missing. Please fill out all fields.', 'ags-dashboard') );
		}
		
		if ( !is_email($_POST['ticket']['email']) ) {
			wp_send_json_error( __('Please enter a valid email address.', 'ags-dashboard') );
		}
		
		$emailAddress = sanitize_text_field($_POST['ticket']['email']);
		
		$email = new AGSDashboard_Support_Email(
			sanitize_text_field($_POST['ticket']['name']),
			$emailAddress,
			sanitize_text_field($_POST['ticket']['product']),
			sanitize_text_field($_POST['ticket']['subject']),
			sanitize_text_field($_POST['ticket']['message']),
			!empty($_POST['ticket']['systemInfo']) && !empty($_POST['ticket']['systemInfoContent']) ? sanitize_textarea_field($_POST['ticket']['systemInfoContent']) : null
		);
		
		$emailResult = $email->send();
		
		if (!$emailResult) {
			$errorMessage = $email->getErrorMessage();
			wp_send_json_error( __('The email failed to send.', 'ags-dashboard').( $errorMessage ? ' '.$errorMessage : '' ) );
		}
		
		if ( !empty($_POST['ticket']['supportAccess']) ) {
			try {
				self::setRemoteAccess( true, $emailAddress );
			} catch (Exception $ex) {}
		}
		
		wp_send_json_success( [ 'remoteAccessStatus' => self::getRemoteAccessStatus() ] );
		
	}
	
	static function computeRelevance($rank, $relevanceInfo, $isInstalled) {
		if ($isInstalled) {
			return -1;
		}
		
		$relevance = $rank;
		foreach ($relevanceInfo as $relevanceItem) {
			if ( isset($relevanceItem['type']) && isset($relevanceItem['value'])  && isset($relevanceItem['points']) ) {
				switch ($relevanceItem['type']) {
					case 'hasTheme':
						if (
							is_network_admin()
								? in_array( strtolower( $relevanceItem['value'] ), array_map( 'strtolower', array_keys( wp_get_themes() ) ) )
								: (
									!strcasecmp( get_template(), $relevanceItem['value'] )
									|| !strcasecmp( get_stylesheet(), $relevanceItem['value'] )
									|| !strcasecmp( apply_filters('divi_ghoster_ghosted_theme', ''), $relevanceItem['value'] )
								)
						) {
							$relevance += $relevanceItem['points'];
						}
						break;
						
					case 'hasPlugin':
						if (
							in_array( strtolower( $relevanceItem['value'] ), array_map( 'strtolower', is_network_admin() ? array_keys( get_plugins() ) : get_option('active_plugins') ) )
						) {
							$relevance += $relevanceItem['points'];
						}
						break;
					
					case 'hasLicenseKeyName':
						if ( !is_network_admin() && in_array( strtolower($relevanceItem['value']), array_map( 'strtolower', array_column( self::getCurrentUserLicenseKeys(), 'title' ) ) ) ) {
							$relevance += $relevanceItem['points'];
						}
						break;
				}
			}
		}
		
		return $relevance;
	}
	
}

class AGSDashboard_Support_Email {
	
	private $name, $email, $subject, $message, $systemInfo, $errorMessage;
	
	function __construct($name, $email, $product, $subject, $message, $systemInfo) {
		
		$this->name = $name;
		$this->email = $email;
		$this->subject = '['.$product.'] '.$subject;
		$this->message = $message;
		$this->systemInfo = $systemInfo;
		
	}
	
	function send() {
		if ($this->systemInfo) {
			$uploadDir = wp_upload_dir(null, false);
			if (!empty($uploadDir['basedir'])) {
				$systemInfoFile = $uploadDir['basedir'].'/system-info-'.wp_generate_password(10, false).'.txt';
				file_put_contents($systemInfoFile, $this->systemInfo);
			}
		}
		
		add_filter('wp_mail_from_name', [$this, 'getFromName']);
		
		add_action('wp_mail_failed', [$this, 'handleEmailFailure']);
		
		$result = wp_mail(
			AGSDashboard::SUPPORT_EMAIL,
			$this->subject,
			$this->message,
			'Reply-To: '.$this->name.' <'.$this->email.'>',
			isset($systemInfoFile) ? [$systemInfoFile] : []
		);
		
		remove_filter('wp_mail_from_name', [$this, 'getFromName']);
		
		@unlink($systemInfoFile);
		
		return $result;
	}
	
	function getFromEmail() {
		return $this->email;
	}
	
	function getFromName() {
		return $this->name;
	}
	
	function handleEmailFailure($error) {
		if ( is_wp_error($error) ) {
			$this->errorMessage = $error->get_error_message();
		}
	}
	
	function getErrorMessage() {
		return $this->errorMessage;
	}
	
}

AGSDashboard::setup();

register_activation_hook(__FILE__, ['AGSDashboard', 'onPluginActivate']);
register_deactivation_hook(__FILE__, ['AGSDashboard', 'onPluginDeactivate']);

}
