<?php
AGSDashboard::VERSION; // Access control

class AGSDashboardAccount {
	private $site, $api, $lastLoginError, $isLoginLocked;
	
	public function __construct($site) {
		if ( $site != 'divispace' ) {
			throw new Exception('Unknown site');
		}
		$this->site = $site;
		
		include_once(__DIR__.'/api.php');
		$this->api = new AGSDashboardApi($this);
	}
	
	public function login($email, $password) {
		$loginResult = $this->api->get_auth_token(
			array(
				'authTokenEmail' => $email,
				'authTokenPassword' => $password,
				'authTokenSite' => get_option('siteurl')
			)
		);
		
		if (!empty($loginResult['success']) && !empty($loginResult['data']['token'])) {
			$tokenUserId = get_current_user_id();
			return update_user_meta(
				$tokenUserId,
				'_ags_dashboard_auth_'.$this->site,
				array(
					'email' => $email,
					'token' => $loginResult['data']['token']
				)
			);
		}
		
		$this->lastLoginError = empty($loginResult['success']) && !empty($loginResult['data']) ? $loginResult['data'] : '';
		$this->isLoginLocked = ($this->lastLoginError == 'temp_locked');
		
		return false;
	}
	
	function getLastLoginError() {
		if ( $this->lastLoginError ) {
			switch ( $this->lastLoginError ) {
				case 'auth':
					return __('Invalid email or password', 'ags-dashboard');
				case 'temp_locked':
					return __('Temporarily locked; try again in 5 minutes', 'ags-dashboard');
				default:
					return __('Unknown error', 'ags-dashboard');
			}
		}
		
		return '';
	}
	
	function isLoginLocked() {
		return $this->isLoginLocked;
	}
	
	function getSiteId() {
		return $this->site;
	}
	
	function getApi() {
		return $this->api;
	}
	
	public function getToken() {
		
		$tokenUserId = get_current_user_id();
		$auth = get_user_meta(
			$tokenUserId,
			'_ags_dashboard_auth_'.$this->site,
			true
		);
		
		if (!empty($auth)) {
			return $auth['token'];
		}
		
	}
	public function getAccountEmail() {
		$tokenUserId = get_current_user_id();
		$auth = get_user_meta(
			$tokenUserId,
			'_ags_dashboard_auth_'.$this->site,
			true
		);
		
		if (!empty($auth)) {
			return $auth['email'];
		}
	}
	
	public function isLoggedIn() {
		$token = $this->getToken();
		return !empty($token);
	}
	
	public function logout() {
		$logoutResult = $this->api->cancel_auth_token();
		
		$tokenUserId = get_current_user_id();
	
		return
			delete_user_meta($tokenUserId, '_ags_dashboard_auth_'.$this->site)
			&& !empty($logoutResult['success']);
	}
}