<?php
AGSDashboard::VERSION; // Access control

$mergedCatalog = AGSDashboard::getMergedCatalog();

$membershipProducts = self::getMembershipProducts();

foreach ( self::getInstalledProducts(['name']) as $installedProduct ) {
	foreach ($mergedCatalog as &$catalogProduct) {
		$isMatchingProduct = array_intersect_assoc( $catalogProduct['ids'], $installedProduct['ids']);
		
		if ($isMatchingProduct) {
			$catalogProduct['installedIds'] = $catalogProduct['ids'];
		} else if ( !empty( $catalogProduct['alternateIds'] ) ) {
			foreach ( $catalogProduct['alternateIds'] as $siteId => $alternateIds ) {
				foreach ($alternateIds as $alternateId) {
					$isMatchingProduct = array_intersect_assoc( [ $siteId => $alternateId ], $installedProduct['ids'] );
					if ($isMatchingProduct) {
						$catalogProduct['installedIds'] = [ $siteId => $alternateId ];
						break 2;
					}
				}
			}
		}
		
		if ( $isMatchingProduct ) {
			$catalogProduct = array_merge( $catalogProduct, $installedProduct );
			$catalogProduct['installed'] = true;
			$catalogProduct['installedFile'] = $catalogProduct['file'];
			continue 2;
		}
	}
	$mergedCatalog[] = [
		'ids' => $installedProduct['ids'],
		'installedIds' => $installedProduct['ids'],
		'name' => $installedProduct['name'],
		'author' => '',
		'type' => $installedProduct['type'],
		'file' => $installedProduct['file'],
		'installedFile' => $installedProduct['file'],
		'description' => '',
		'url' => '',
		'installed' => true
	];
}

$installedPlugins = get_plugins();
$installedThemes = wp_get_themes();


foreach ($mergedCatalog as &$catalogItem) {
	if ( empty($catalogItem['installed']) ) {
		switch ( $catalogItem['type'] ) {
			case 'plugin':
				if (isset($installedPlugins[ $catalogItem['file'] ])) {
					$catalogItem['installed'] = true;
					$catalogItem['noActivation'] = true;
					$catalogItem['installedIds'] = $catalogItem['ids'];
					$catalogItem['installedFile'] = $catalogItem['file'];
					$catalogItem['active'] = is_plugin_active($catalogItem['file']);
					$hasInstalledProduct = true;
				}
				break;
			case 'theme':
				$themeSlug = dirname($catalogItem['file']);
				if (isset($installedThemes[ $themeSlug ])) {
					$catalogItem['installed'] = true;
					$catalogItem['noActivation'] = true;
					$catalogItem['installedIds'] = $catalogItem['ids'];
					$catalogItem['installedFile'] = $catalogItem['file'];
					$catalogItem['active'] = get_stylesheet() === $themeSlug;
					$hasInstalledProduct = true;
				}
				break;
		}
	} else {
		$hasInstalledProduct = true;
		if (!in_array(basename(dirname($catalogItem['installedFile'])).'/'.basename($catalogItem['installedFile']), self::$productsWithUpdaters)) {
			$catalogItem['noActivation'] = true;
		}
	}

	$catalogItem['relevance'] = isset($catalogItem['relevance'])
		? self::computeRelevance(
			isset($catalogItem['rank']) ? $catalogItem['rank'] : 0,
			$catalogItem['relevance'],
			// If the product was just installed in the current request, compute relevance as if it were not installed to avoid it disappearing from the current catalog page
			// phpcs:ignore WordPress.Security.NonceVerification -- we are basically just checking to see if we are currently in an install request
			!empty( $catalogItem['installed'] ) && ( empty($_GET['id']) || !in_array( $_GET['id'], $catalogItem['installedIds'] ) || empty($_GET['action']) || $_GET['action'] != 'ags_dashboard_install' )
		)
		: 0;
}

usort($mergedCatalog, function($item1, $item2) {
	return $item2['relevance'] - $item1['relevance'];
});

$allCategories = array_unique( array_reduce( array_column( $mergedCatalog, 'categories' ), 'array_merge', [] ) );
sort($allCategories);

$fixedCategoryOrder = [
	'Plugins',
	'Child Themes',
	'Membership'
];
$allCategories = array_merge( array_intersect($fixedCategoryOrder, $allCategories), array_diff($allCategories, $fixedCategoryOrder) );

$allTags = array_unique( array_reduce( array_column( $mergedCatalog, 'tags' ), 'array_merge', [] ) );
sort($allTags);

$isNetwork = is_multisite();
$isNetworkAdmin = is_network_admin();
$disableNetworkAdminFeatures = $isNetwork && !$isNetworkAdmin && !$forceIsNetworkAdmin;

$statusFilteringOptions = [
	'purchase' => __('Available to purchase', 'ags-dashboard'),
	'installable' => __('Available to install', 'ags-dashboard'),
	'installed' => __('Installed', 'ags-dashboard'),
	'active' => __('Activated', 'ags-dashboard'),
	'free' => __('Free', 'ags-dashboard')
];

?>

<?php if (!wp_doing_ajax()) { ?>

<?php AGSDashboard::showAdminNavbar('products'); ?>

<div class="ags-dashboard-settings-wrapper">

<?php if (isset($_GET['needsActivation'])) { // phpcs:ignore WordPress.Security.NonceVerification -- read-only ?>
<div class="ags-dashboard-notice notice notice-error">
	<p>
	<?php printf(
		// Translators: %s is the product name
		esc_html__('%s does not currently have an active license key. Please activate your license key below.', 'ags-dashboard'),
		// phpcs:ignore WordPress.Security.NonceVerification -- read-only
		( (bool) $_GET['needsActivation'] ? '<strong>'.esc_html( sanitize_text_field( $_GET['needsActivation'] ) ).'</strong>' : esc_html__('This product', 'ags-dashboard') )
	); ?>
	</p>
</div>
<?php } ?>
    <div class="ags-dashboard-product-catalog-wrapper">

		<div class="ags-dashboard-product-catalog-options-wrapper">
            <input id="ags-dashboard-product-catalog-search" type="search" placeholder="<?php esc_attr_e('Search Products By Name', 'ags-dashboard'); ?>"<?php if (!empty($_GET['needsActivation'])) { echo( ' value="'.esc_attr( sanitize_text_field( $_GET['needsActivation'] ) ).'"' ); } // phpcs:ignore WordPress.Security.NonceVerification -- read-only ?>>

            <ul id="ags-dashboard-product-catalog-filtering" class="ags-dashboard-tabs">
                <li class="ags-dashboard-active"><a href="#" data-ags-dashboard-category="*"><?php esc_html_e('All', 'ags-dashboard'); ?></a></li>
                <?php foreach ( $allCategories as $category ) { ?>
                    <li><a href="#" data-ags-dashboard-category="<?php echo( esc_attr( sanitize_key( $category ) ) ); ?>"><?php echo( esc_html( $category ) ); ?></a></li>
                <?php } ?>
            </ul>

            <div id="ags-dashboard-product-catalog-filtering-dropdown" class="ags-dashboard-dropdown">
                <a href="#">Category</a>
                <ul class="ags-dashboard-checkbox-menu">
                    <li>
                        <span><?php esc_html_e('All', 'ags-dashboard'); ?></span>
                        <input type="radio" name="ags-dashboard-category" value="*" checked>
                    </li>
                   <?php foreach ( $allCategories as $category ) { ?>
                    <li>
                        <span><?php echo( esc_html( $category ) ); ?></span>
                        <input type="radio" name="ags-dashboard-category" value="<?php echo( esc_attr( sanitize_key( $category ) ) ); ?>">
                    </li>
                    <?php } ?>
                </ul>
            </div>

            <ul id="ags-dashboard-product-catalog-status-options" class="ags-dashboard-tabs">
                <?php foreach ($statusFilteringOptions as $statusId => $statusName) { ?>
                <li>
                    <a href="#" data-ags-dashboard-status="<?php echo( esc_attr($statusId) ); ?>"><?php echo( esc_html($statusName) ); ?></a>
                </li>
                <?php } ?>
            </ul>

            <div id="ags-dashboard-product-catalog-status-options-dropdown" class="ags-dashboard-dropdown">
                <a href="#">Status</a>
                <ul class="ags-dashboard-checkbox-menu">
                    <?php foreach ($statusFilteringOptions as $statusId => $statusName) { ?>
                    <li>
                        <span><?php echo( esc_html($statusName) ); ?></span>
                        <input type="checkbox" value="<?php echo( esc_attr($statusId) ); ?>">
                    </li>
                    <?php } ?>
                </ul>
            </div>

            <div id="ags-dashboard-product-catalog-tag-options" class="ags-dashboard-dropdown">
                <a href="#">Tags</a>
                <ul class="ags-dashboard-checkbox-menu">
                    <?php foreach ($allTags as $productTag) { ?>
                    <li>
                        <span><?php echo( esc_html($productTag) ); ?></span>
                        <input type="checkbox" value="<?php echo( esc_attr( sanitize_key($productTag) ) ); ?>">
                    </li>
                    <?php } ?>
                </ul>
            </div>
		</div>


        <div id="ags-dashboard-product-catalog" class="ags-dashboard-block-content"  data-ags-dashboard-per-page="6">

<?php } // doing ajax check ?>

        <?php
            if (empty($mergedCatalog)) {
        ?>
            <p class="ags-dashboard-nothing">
                <?php esc_html_e('We\'re currently having trouble retrieving the product catalog. Please try again soon.', 'ags-dashboard'); ?>
            </p>
        <?php
            } else {

				$ajaxUrl = admin_url('admin-ajax.php');

                foreach ( $mergedCatalog as $product ) {
					unset($name, $licenseKeyInfo, $installUrl);
					$isSelf = $product['ids'] == self::OWN_PRODUCT_ID;
					
					if (!empty($product['installedFile'])) {
						$name = $product['type'] == 'plugin' ? plugin_basename( $product['installedFile'] ) : basename( dirname( $product['installedFile'] ) );
					} else if (!empty($product['file'])) {
						$name = $product['type'] == 'plugin' ? plugin_basename( $product['file'] ) : basename( dirname( $product['file'] ) );
					}


                    $itemClasses = 'ags-dashboard-catalog-item';

					if ( empty($product['installed']) ) {

						if ( empty($product['file']) || $disableNetworkAdminFeatures ) {
							$installUrl = 'na';
						} else {

							if (empty($product['dotOrg'])) {
								$licenseKeys = self::getCurrentUserLicenseKeysByProductIds($product['ids']);
								foreach ($licenseKeys as $licenseKeyData) {
									if ( $licenseKeyData['status'] == 'active' || $licenseKeyData['status'] == 'inactive' ) {
										$installUrl = add_query_arg(
											[
												'action' => 'ags_dashboard_install',
												'id' => $product['ids'][ $licenseKeyData['site'] ],
												'site' => $licenseKeyData['site'],
												'key' => $licenseKeyData['key'],
												'type' => $product['type'],
												'nonce' => wp_create_nonce( 'ags_dashboard_install_'.$licenseKeyData['site'].$product['ids'][ $licenseKeyData['site'] ] )
											],
											$ajaxUrl
										);
										break;
									}
								}
							} else {
								$installUrl = add_query_arg(
									[
										'action' => 'ags_dashboard_install',
										'id' => dirname($product['file']),
										'site' => 'wordpress.org',
										'type' => $product['type'],
										'nonce' => wp_create_nonce( 'ags_dashboard_install_wordpress.org'.dirname($product['file']) )
									],
									$ajaxUrl
								);
							}
						}


						if ( isset($installUrl) ) {
							if ( !empty($product['file']) ) {
								$itemClasses .= ' ags-dashoard-catalog-item-status-installable';
							}
						} else {
							$itemClasses .= ' ags-dashoard-catalog-item-status-purchase';
						}

					} else {


						$itemClasses .= ' ags-dashoard-catalog-item-status-installed';
						if (!$isNetworkAdmin) {
							$itemClasses .= ( empty($product['active']) ? ' ags-dashoard-catalog-item-status-inactive' : ' ags-dashoard-catalog-item-status-active' );

							if ( empty($product['noActivation']) && empty($product['dotOrg']) && is_numeric( current( $product['installedIds'] ) ) && !$isSelf ) {
								$licenseKeyInfo = self::getProductLicenseKeyInfo( $product['installedIds'] );
								$itemClasses .= ' ags-dashboard-key-status-'.$licenseKeyInfo['status'];
							}
						}

                    }

					if (!empty($product['dotOrg'])) {
						$itemClasses .= ' ags-dashoard-catalog-item-status-free';
					}

                    if (isset($product['categories'])) {
                        foreach ( $product['categories'] as $category ) {
                            $itemClasses .= ' ags-dashoard-catalog-item-category-'.sanitize_key( $category );
                        }
                    }

                    if (isset($product['tags'])) {
                        foreach ( $product['tags'] as $productTag ) {
                            $itemClasses .= ' ags-dashoard-catalog-item-tag-'.sanitize_key( $productTag );
                        }
                    }
        ?>
                <div class="<?php echo( esc_attr( $itemClasses ) ); ?>" data-ags-dashboard-item-name="<?php echo( esc_attr( strtolower( $product['name'] ) ) ); ?>">
                    <div class="ags-dashboard-product-flex-container">
                        <div class="ags-dashboard-product-details">

                            <div class="ags-dashboard-product-image">
                                <?php if (!empty($product['thumbnails']['list'])) { ?>
                                    <img class="ags-dashboard-product-thumbnail-catalog" src="<?php echo( esc_url( $product['thumbnails']['list'] ) ); ?>">
                                <?php } else { ?>
                                    <img class="ags-dashboard-product-thumbnail-catalog" src="<?php echo( esc_url(self::$pluginBaseUrl . 'images/placeholder_featured_image.jpg') ); ?>">
                                <?php } ?>
                            </div>

                            <div class="ags-dashboard-product-title">
                                <strong class="ags-dashboard-product-name">
                                    <?php echo( ( empty($_GET['relDebug']) ? '' : '('.(int) $product['relevance'].') ' ).esc_html( $product['name'] ) ); // phpcs:ignore WordPress.Security.NonceVerification -- read-only ?>
                                </strong>
                                <p class="ags-dashboard-product-author">
                                    <?php
                                    echo(
                                        esc_html(
                                            sprintf(
                                                // Translators: %s is product author
                                                'by %s',
                                                empty($product['author']) ? __('Unknown', 'ags-dashboard') : $product['author']
                                            )
                                        )
                                    );
                                    ?>
                                </p>

                            </div>

                            <p class="ags-dashboard-product-description">
                                <?php if (!empty($product['description'])) { echo( esc_html( $product['description'] ) ); } ?>
                            </p>

                        </div>

                        <div class="ags-dashboard-product-actions">

							<ul class="ags-dashboard-product-status">
								<?php
									if (empty($product['dotOrg']) && !$isSelf) {
										$keys = self::getCurrentUserLicenseKeysByProductIds($product['ids']);
										if (!$keys) {
											$availabilityClasses = '';
											$availabilityText = __('Please purchase to install', 'ags-dashboard');
										} else {
											$availabilityClasses = ' ags-dashboard-active';
											$availabilityText = __('Available', 'ags-dashboard');
											foreach ( $keys as $keyData ) {
												if (count($keyData['products']) > 5) {
													$availabilityClasses .= ' ags-dashboard-membership';
													$availabilityText = __('Available in your membership', 'ags-dashboard');
													break;
												}
											}
										}
									} else {
										$availabilityClasses = ' ags-dashboard-active ags-dashboard-free-product';
										$availabilityText = __('Available - free product!', 'ags-dashboard');
									}

								?>
								<li class="ags-dashboard-tooltip ags-dashboard-product-status-availability<?php echo( esc_attr($availabilityClasses) ); ?>">
									<span class="ags-dashboard-tooltip-text">
                                        <?php echo(esc_html($availabilityText)); ?>
                                    </span>
                                    <?php
                                    if ($availabilityText === "Please purchase to install") {
                                        echo file_get_contents(self::$pluginDirectory . 'images/svg/lock.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                                    } else {
                                        echo file_get_contents(self::$pluginDirectory . 'images/svg/lock-on.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                                    } ?>
								</li>
								<li class="ags-dashboard-tooltip ags-dashboard-product-status-installation<?php if ( !empty($product['installed']) ) echo(' ags-dashboard-active'); ?>">
									<span class="ags-dashboard-tooltip-text">
                                        <?php empty($product['installed']) ? esc_html_e('Not installed', 'ags-dashboard') : esc_html_e('Installed', 'ags-dashboard'); ?>
                                    </span>
                                    <?php echo file_get_contents(self::$pluginDirectory . 'images/svg/download.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
								</li>
								<?php if (!$isNetworkAdmin) { ?>
									<li class="ags-dashboard-tooltip ags-dashboard-product-status-activation<?php if ( !empty($product['active']) ) echo(' ags-dashboard-active'); ?>">
                                        <span class="ags-dashboard-tooltip-text">
										    <?php empty($product['active']) ? esc_html_e('Inactive', 'ags-dashboard') : esc_html_e('Active', 'ags-dashboard'); ?>
                                        </span>
                                        <?php echo file_get_contents(self::$pluginDirectory . 'images/svg/on.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
									</li>
									<?php
										if (isset($licenseKeyInfo) && !$isSelf) {
											$licenseKeyStatusText = self::getLicenseKeyStatusName($licenseKeyInfo['status'], $licenseKeyInfo['expiration']);

											switch ($licenseKeyInfo['status']) {
												case 'valid':
													$licenseKeyStatusClass = $licenseKeyInfo['expiration'] == 'lifetime' || $licenseKeyInfo['expiration'] > (time() + 86400) ? ' ags-dashboard-active' : ' ags-dashboard-status-warning';
													break;
												case 'none':
													$licenseKeyStatusClass = '';
													break;
												default:
													$licenseKeyStatusClass = ' ags-dashboard-status-error';
											}

										} else {
											$licenseKeyStatusClass = '';
											if (!empty($product['dotOrg']) || $isSelf) {
												$licenseKeyStatusText = __('Not applicable (free product)', 'ags-dashboard');
											} else if (empty($product['installed'])) {
												$licenseKeyStatusText = __('Not applicable (not installed)', 'ags-dashboard');
                                                $licenseKeyStatusClass = ' ags-dashboard-status-not-applicable';
											} else {
												$licenseKeyStatusText = __('Unknown', 'ags-dashboard');
											}
										}
									?>
									<li class="ags-dashboard-manage-license-key-link ags-dashboard-tooltip ags-dashboard-product-status-key<?php echo( esc_attr( $licenseKeyStatusClass ) ); ?>">
                                        <span class="ags-dashboard-tooltip-text">
										    <?php echo( esc_html( $licenseKeyStatusText ) ); ?>
                                        </span>
                                        <span class="ags-dashboard-product-status-key-icon">
                                            <?php echo file_get_contents(self::$pluginDirectory . 'images/svg/key.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
                                        </span>
                                    </li>
								<?php } ?>

							</ul>

                            <div class="ags-dashboard-product-buttons-wrapper">

                                <?php if (!empty($product['docs'])) { ?>
                                    <a href="<?php echo( esc_url( $product['docs'] ) ); ?>" class="button ags-dashboard-btn-outline ags-dashboard-docs-link ags-dashboard-link-external" target="_blank">
                                        <?php esc_html_e('Documentation', 'ags-dashboard'); ?>
                                    </a>
                                <?php } ?>

                                <?php if ( empty($product['installed']) ) { ?>

                                    <?php
                                        if ( isset($installUrl) && $product['type'] != 'membership' ) {
                                            if ( !empty($product['file']) ) {
                                        ?>
                                            <a href="#" class="button ags-dashboard-btn-primary<?php
                                                if ( $disableNetworkAdminFeatures ) {
                                                    echo(' ags-dashboard-refer-network-admin"');
                                                } else {
                                                    echo(' ags-dashboard-ajax-link" data-ags-dashboard-href="'.esc_url($installUrl).'"');
                                                }
                                                ?>">
                                                <?php $isNetwork ? esc_html_e('Install', 'ags-dashboard') : esc_html_e('Install & Activate', 'ags-dashboard'); ?>
                                            </a>
                                        <?php
                                            }
                                        } else {
											if ( $product['type'] == 'membership' ) {
												$membershipKeys = self::getCurrentUserLicenseKeysByName( $product['name'], key( $product['ids'] ) );
												if ($membershipKeys) {
													$membershipKey = current($membershipKeys);
												} else {
													unset($membershipKey);
												}
											}
											if ( $product['type'] != 'membership' || empty($membershipKey) ) {
                                    ?>
                                    <a href="<?php if ( $product['type'] == 'membership' ) { echo( esc_url( self::getAddToCartUrl( $product['ids']['divispace'] ) ) ); } else { echo('#'); } ?>" target="_blank" class="button ags-dashboard-btn-primary ags-dashboard-purchase-link">
                                        <?php esc_html_e('Purchase', 'ags-dashboard'); ?>
                                    </a>
											<?php } ?>
									<?php if ($product['type'] != 'membership') { ?>
                                    <div class="ags-dashboard-purchase-options">
                                        <h3>
                                            <?php echo(
                                                    sprintf(
                                                        // translators: %1s is <span>, %2s is </span>
                                                        esc_html__('Get %1sall of our products%2s for one low price!', 'ags-dashboard'),
                                                       '<span>',
                                                        '</span>'
                                                    )
                                                ); ?>
                                        </h3>
                                        <div class="agsd-row">
                                            <div class="agsd-col-sm-6">
                                                <p><?php esc_html_e('Our membership options give you access to our entire product catalog, integrated with the Helper to provide easy one-click installation.', 'ags-dashboard'); ?></p>

                                                <div class="ags-dashboard-purchase-options-products">
                                                    <?php foreach ( $membershipProducts as $membershipId => $membership ) { ?>
                                                        <div class="ags-dashboard-purchase-options-product">
                                                            <?php if (!empty($membership['thumbnails']['list'])) { ?>
                                                                <img class="ags-dashboard-product-thumbnail-catalog" src="<?php echo( esc_url( $membership['thumbnails']['list'] ) ); ?>">
                                                            <?php } else { ?>
                                                                <img class="ags-dashboard-product-thumbnail-catalog" src="<?php echo( esc_url(self::$pluginBaseUrl . 'images/placeholder_featured_image.jpg') ); ?>">
                                                            <?php } ?>

                                                            <a href="<?php echo( esc_url( self::getAddToCartUrl( substr($membershipId, 1) ) ) ); ?>" target="_blank" class="button ags-dashboard-btn-outline">
                                                                <?php echo( esc_html( $membership['name'] ) ); ?>
                                                            </a>
                                                        </div>
                                                    <?php } ?>
                                                </div>
                                            </div>
                                            <div class="agsd-col-sm-6">
                                                <p><?php esc_html_e('Or, you can purchase this product individually if you prefer.', 'ags-dashboard'); ?></p>

                                                <div class="ags-dashboard-purchase-options-products">
                                                    <div class="ags-dashboard-purchase-options-product">
                                                        <?php if (!empty($product['thumbnails']['list'])) { ?>
                                                            <img class="ags-dashboard-product-thumbnail-catalog" src="<?php echo( esc_url($product['thumbnails']['list'] ) ); ?>">
                                                        <?php } else { ?>
                                                            <img class="ags-dashboard-product-thumbnail-catalog" src="<?php echo( esc_url(self::$pluginBaseUrl . 'images/placeholder_featured_image.jpg') ); ?>">
                                                        <?php } ?>
                                                        <a href="<?php echo( esc_url( $product['type'] == 'theme' ? self::getAddToCartUrl( $product['ids']['divispace'] ) : $product['url'] ) ); ?>" target="_blank" class="button ags-dashboard-btn-secondary">
                                                            <?php esc_html_e('Individual purchase', 'ags-dashboard'); ?>
                                                        </a>
                                                    </div>
                                                </div>
                                            </div>
                                        </div>
                                    </div>
									<?php } ?>
                                    <?php
                                        }
                                    ?>
									
									
								<?php if ( $product['type'] == 'membership' && !empty($membershipKey) && $membershipKey['status'] === 'expired' ) { ?>
									<a href="<?php echo(
										esc_url(
											self::getLicenseKeyRenewalUrl(
												$membershipKey['site'],
												$membershipKey['key']
											)
										)
									); ?>" target="_blank" class="button ags-dashboard-btn-primary ags-dashboard-link-external ags-dashboard-link-renew-license-key"><?php esc_html_e('Renew', 'ags-dashboard'); ?></a>
								<?php } ?>
									
                                <?php
                                    } else {

                                    if (!$isNetworkAdmin) {

                                            switch ( $product['type'] ) {
                                                case 'plugin':
                                                    $activeStatusChangeLabel = empty($product['active']) ? __('Activate Plugin', 'ags-dashboard') : __('Deactivate Plugin', 'ags-dashboard');
                                                    break;
                                                case 'theme':
                                                    if ( empty($product['active']) ) {
                                                        $activeStatusChangeLabel = __('Set as Active Theme', 'ags-dashboard');
                                                    } else {
														unset( $activeStatusChangeLabel );
													}
                                                    break;
                                            }

                                            ?>
                                            <?php if (isset($activeStatusChangeLabel)) { ?>
                                                <a	href="<?php echo( esc_url( self::getChangeActiveStatusUrl($ajaxUrl, $name, $product['type'], !empty($product['active']) ) ) ); ?>"
													class="button ags-dashboard-btn-primary ags-dashboard-ajax-link"
													data-ags-dashboard-status-elem=".ags-dashboard-product-status-activation"
													data-ags-dashboard-status-text-elem=".ags-dashboard-product-status-activation .ags-dashboard-tooltip-text">
														<?php echo( esc_html($activeStatusChangeLabel) ); ?>
                                                </a>
                                            <?php } ?>
											
												
											<?php if (!empty($licenseKeyInfo['key']) && isset( $product['installedIds'][ $licenseKeyInfo['site'] ] )
																&& $licenseKeyInfo['expiration'] !== 'lifetime' && $licenseKeyInfo['expiration'] < time() + (86400 * 14)
											) { ?>
												<a href="<?php echo(
													esc_url(
														self::getLicenseKeyRenewalUrl(
															$licenseKeyInfo['site'],
															$licenseKeyInfo['key']
														)
													)
												); ?>" target="_blank" class="button ags-dashboard-btn-primary ags-dashboard-link-external ags-dashboard-link-renew-license-key"><?php esc_html_e('Renew', 'ags-dashboard'); ?></a>
											<?php } ?>


                                        <?php } ?>

                                <?php } ?>
                            </div>
                        </div>
                    </div>

                    <div class="ags-dashboard-dropdown ags-dashboard-more">
                        <a href="#"><?php esc_html_e('More', 'ags-dashboard'); ?></a>
                        <ul>

                            <?php if (!$isNetworkAdmin) { ?>
                                <li>
                                    <a href="?page=ags-dashboard-support&q=<?php echo( rawurlencode( $product['name'] ) ); ?>" class="ags-dashboard-support-link"><?php esc_html_e('Support', 'ags-dashboard'); ?></a>
                                </li>

								<?php if ( isset($licenseKeyInfo) ) { ?>
									<li>
										<a href="<?php echo( esc_url(
													add_query_arg(
														[
															'action' => 'ags_dashboard_license_key_form',
															'type' => $product['type'],
															'name' => $name
														],
														$ajaxUrl
													)
												) ); ?>" class="ags-dashboard-manage-license-key-link">
											<?php esc_html_e('Manage License Key', 'ags-dashboard'); ?>
										</a>
									</li>
								<?php } ?>

								<?php if (!empty($product['active']) && !empty($product['adminPage'])) { ?>
									<li>
										<a href="<?php echo( esc_url( admin_url( $product['adminPage'] ) ) ); ?>" class="ags-dashboard-admin-page-link">
											<?php esc_html_e('Admin Page', 'ags-dashboard'); ?>
										</a>
									</li>
								<?php } ?>

                            <?php } ?>

                            <?php if (!empty($product['url'])) { ?>
                                <li>
                                    <a href="<?php echo( esc_url( $product['url'] ) ); ?>" target="_blank" class="ags-dashboard-link-external ags-dashboard-link-product-page"><?php esc_html_e('Product Page', 'ags-dashboard'); ?></a>
                                </li>
                            <?php } ?>

                            <?php if ( !empty($product['url']) && ($product['type'] == 'plugin' || $product['type'] == 'theme') ) { ?>
                                <li>
                                    <a href="<?php echo( esc_url( add_query_arg('changelog', 1, $product['url']) ) ); ?>" target="_blank" class="ags-dashboard-link-external ags-dashboard-link-changelog"><?php esc_html_e('Changelog', 'ags-dashboard'); ?></a>
                                </li>
                            <?php } ?>



                            <?php if ( !$isNetworkAdmin && !empty($licenseKeyInfo['key']) ) { ?>
                                <li>
                                    <a href="<?php echo(
                                    esc_url(
                                        add_query_arg(
                                            [
                                                'ags_dashboard_redirect' => 'manageSites',
                                                'license_key' => $licenseKeyInfo['key']
                                            ],
                                            self::SITE_URL
                                        )
                                    )
                                    ); ?>" target="_blank" class="ags-dashboard-link-external ags-dashboard-link-manage-activations"><?php esc_html_e('Manage Activations', 'ags-dashboard'); ?></a>
                                </li>
                            <?php } ?>

                            <?php if ( !$isNetworkAdmin && !empty($licenseKeyInfo['key']) && isset( $product['installedIds'][ $licenseKeyInfo['site'] ] ) ) { ?>
                                <li>
                                    <a href="<?php echo(
                                    esc_url(
                                        self::getLicenseKeyRenewalUrl($licenseKeyInfo['site'], $licenseKeyInfo['key'])
                                    )
                                    ); ?>" target="_blank" class="ags-dashboard-link-external ags-dashboard-link-renew-license-key"><?php esc_html_e('Renew License Key', 'ags-dashboard'); ?></a>
                                </li>
                            <?php } ?>
                        </ul>
                    </div>

                    <div class="ags-dashboard-product-badges-wrapper">
                        <?php
                        if (isset($product['badges'])) {
                            foreach ( $product['badges'] as $badge ) {
                                if ( empty( $badge['label'] ) ) {
                                    continue;
                                }
                                $badgeStyle = ( isset($badge['textColor']) ? 'color: '.$badge['textColor'].';' : '' ).( isset($badge['bgColor']) ? 'background: '.$badge['bgColor'].';' : '' );
                                $badgeShadowStyle = ( isset($badge['shadowColor']) ? 'border-bottom-color: '.$badge['shadowColor'].';' : '' );
                                ?>
                                <div class="ags-dashboard-product-badge"<?php if (!empty($badgeStyle)) { echo(' style="'.esc_attr($badgeStyle).'"'); } ?>>
                                    <?php echo( esc_html( $badge['label'] ) ); ?>
                                    <span class="ags-dashboard-product-badge-shadow" <?php if (!empty($badgeShadowStyle)) { echo(' style="'.esc_attr($badgeShadowStyle).'"'); } ?>></span>
                                </div>
                                <?php
                            }
                        }
                        ?>
                    </div>
                </div>
            <?php } ?>
        <?php
            }
        ?>

<?php if (!wp_doing_ajax()) { ?>
        </div>
    </div>

</div>

<?php } // doing ajax check