<?php
AGSDashboard::VERSION; // Access control

$products = self::getInstalledProducts(['name', 'thumbnails']);
$canDoRemoteAccess = current_user_can('create_users');
?>

<?php AGSDashboard::showAdminNavbar('support'); ?>

<div class="ags-dashboard-settings-wrapper">

    <div id="ags-dashboard-support-articles" class="ags-dashboard-block">
        <div class="ags-dashboard-block-title">
            <h2><?php esc_html_e('Find your answers here', 'ags-dashboard'); ?></h2>
        </div>
        <div class="ags-dashboard-block-content">
            <div class="ags-dashboard-support-search-wrapper">
                <input type="search" placeholder="<?php esc_attr_e('Search our knowledgebase...', 'ags-dashboard'); ?>"<?php
                // phpcs:ignore WordPress.Security.NonceVerification -- read-only
                if (isset($_GET['q'])) {
                    // phpcs:ignore WordPress.Security.NonceVerification -- read-only
                    echo(' value="' . esc_attr(sanitize_text_field($_GET['q'])) . '"');
                }
                ?>>
            </div>
            <div id="ags-dashboard-support-articles-list-container">
                <div class="ags-dashboard-loading"><span><?php esc_html_e('Loading...', 'ags-dashboard'); ?></span></div>
            </div>
        </div>
    </div>

    <?php if (array_column($products, 'docs')) { ?>
        <div id="ags-dashboard-support-documentation" class="ags-dashboard-block ags-dashboard-block-grey">
            <div class="ags-dashboard-block-title">
                <h2><?php esc_html_e('Product documentation', 'ags-dashboard'); ?></h2>
            </div>
            <div id="ags-dashboard-support-documentation-products" class="ags-dashboard-block-content">
                <?php
                foreach ($products as $product) {
                    if (!empty($product['docs'])) {
                        ?>

                        <div class="ags-dashboard-docs-item">
                            <div class="ags-dashboard-product-image">
                                <?php if (!empty($product['thumbnails']['catalog'])) { ?>
                                    <img class="ags-dashboard-product-thumbnail" src="<?php echo(esc_url($product['thumbnails']['list'])); ?>">
                                <?php } ?>
                            </div>
                            <p class="ags-dashboard-product-title">
                                <strong class="ags-dashboard-product-name">
                                    <?php echo(esc_html($product['name'])); ?>
                                </strong>
                            </p>
                            <p class="ags-dashboard-product-actions">
                                <a href="<?php echo(esc_url($product['docs'])); ?>" target="_blank" class="ags-dashboard-docs-link">
                                    <?php esc_html_e('Documentation', 'ags-dashboard'); ?>
                                </a>
                            </p>
                        </div>
                        <?php
                    }
                }
                ?>
            </div>
        </div>
    <?php } ?>

    <div id="ags-dashboard-support-tickets" class="ags-dashboard-block">
        <div class="ags-dashboard-block-title">
            <h2><?php esc_html_e('Open a ticket', 'ags-dashboard'); ?></h2>
        </div>
        <div class="ags-dashboard-block-content">

            <form id="ags-dashboard-support-ticket-form" action="<?php echo(esc_url(add_query_arg('action', 'ags_dashboard_open_support_ticket', admin_url('admin-ajax.php')))); ?>">
                <?php wp_nonce_field('ags_dashboard_open_support_ticket', 'nonce', false); ?>

                <div class="agsd-row">
                    <label class="agsd-col-md-6">
                        <span><?php esc_html_e('Name:', 'ags-dashboard'); ?></span>
                        <input type="text" name="ticket[name]">
                    </label>

                    <label class="agsd-col-md-6">
                        <span><?php esc_html_e('Email:', 'ags-dashboard'); ?></span>
                        <?php
						$account = self::getCurrentUserAccount();
						if ($account->isLoggedIn()) {
							$loggedInEmail = $account->getAccountEmail();
						}
                        if (empty($loggedInEmail)) {
                            ?>
                            <input type="email" name="ticket[email]">
                        <?php } else { ?>
                            <select name="ticket[email]" class="agsd-select-fw">
                                <option value="<?php echo(esc_attr($loggedInEmail)); ?>"><?php echo(esc_html($loggedInEmail)); ?></option>
                            </select>
                        <?php } ?>
                    </label>
                </div>

                <div class="agsd-row">
                    <label class="agsd-col-md-6">
                        <span><?php esc_html_e('Product:', 'ags-dashboard'); ?></span>
                        <select class="agsd-select-fw" name="ticket[product]">
                            <?php foreach (self::getInstalledProducts(['name']) as $product) { ?>
                                <option value="<?php echo(esc_attr($product['name'])); ?>"><?php echo(esc_html($product['name'])); ?></option>
                            <?php } ?>
                            <option value="Other"><?php esc_html_e('Other', 'ags-dashboard'); ?></option>
                        </select>
                    </label>

                    <label class="agsd-col-md-6">
                        <span><?php esc_html_e('Subject:', 'ags-dashboard'); ?></span>
                        <input type="text" name="ticket[subject]">
                    </label>
                </div>

                <div class="agsd-row">
                    <label class="agsd-col-12">
                        <span><?php esc_html_e('Message:', 'ags-dashboard'); ?></span>
                        <textarea name="ticket[message]"></textarea>
                    </label>
                </div>
                <div class="agsd-row">
                    <div class="agsd-col-12">
                        <div class="ags-dashboard-form-checkbox">
                            <label>
                                <input type="checkbox" name="ticket[systemInfo]" checked>
                                <span><?php esc_html_e('Include system information', 'ags-dashboard'); ?></span>
                            </label>

                            <p class="agsd-description"><?php printf(
                                // Translators: %s are <a></a> tags
                                    esc_html__('%sClick here%s to see the information that will be sent.', 'ags-dashboard'),
                                    '<a href="#" id="ags-dashboard-system-info-link">',
                                    '</a>'
                                ); ?></p>
                        </div>
                    </div>
                </div>

                <?php
                // Contains code copied from wp-admin\site-health-info.php
                if (!class_exists('WP_Debug_Data')) {
                    require_once ABSPATH . 'wp-admin/includes/class-wp-debug-data.php';
                }
                WP_Debug_Data::check_for_updates();
                ?>
                <textarea id="ags-dashboard-system-info" name="ticket[systemInfoContent]" class="hidden"><?php echo esc_textarea(trim(trim(WP_Debug_Data::format(WP_Debug_Data::debug_data(), 'debug'), '`'))); ?></textarea>

                <div class="agsd-row">
                    <div class="agsd-col-12">
                        <div class="ags-dashboard-form-checkbox">
                            <label>
                                <input type="checkbox" name="ticket[supportAccess]"<?php if (!$canDoRemoteAccess) echo(' disabled'); ?>>
                                <span><?php esc_html_e('Allow support staff to access your site', 'ags-dashboard'); ?></span>
                            </label>
                            <p class="agsd-description"><?php esc_html_e('Allowing acccess may result in a quicker resolution. Access will automatically expire within 7 days and you can revoke access manually at any time.', 'ags-dashboard'); ?></p>
                        </div>
                        <?php if (!$canDoRemoteAccess) { ?>
                            <p class="ags-dashboard-warning"><?php esc_html_e('Your account does not have sufficient permissions to enable support access. You can still submit a ticket without enabling support access.', 'ags-dashboard'); ?></p>
                        <?php } ?>
                    </div>
                </div>

                <button class="button ags-dashboard-btn-primary"><?php esc_html_e('Open ticket', 'ags-dashboard'); ?></button>

            </form>
        </div>
    </div>

    <?php if ($canDoRemoteAccess) { ?>

        <div id="ags-dashboard-support-access" class="ags-dashboard-block ags-dashboard-block-grey">
            <div class="ags-dashboard-block-title">
                <h2><?php esc_html_e('Support access', 'ags-dashboard'); ?></h2>
            </div>
            <div class="ags-dashboard-block-content">
                <p><?php esc_html_e('To expedite your support request, you may choose to grant temporary Administrator-level access to our support staff. This feature is secure and entirely optional, but it may help us help you more quickly. Once granted, this access will automatically expire within 7 days unless you renew it.', 'ags-dashboard'); ?></p>


                <?php
                $remoteStatus = self::getRemoteAccessStatus()
                ?>

                <p id="ags-dashboard-support-access-status" class="ags-dashboard-support-access-<?php echo($remoteStatus['status'] ? 'enabled' : 'disabled'); ?>">
                    <?php
                    echo file_get_contents(self::$pluginDirectory . 'images/svg/support.svg'); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                    echo(esc_html($remoteStatus['statusText']));
                    ?>
                </p>

                <form action="<?php echo(
                esc_url(
                    add_query_arg(
                        [
                            'action' => 'ags_dashboard_set_remote_access',
                            'nonce'  => wp_create_nonce('ags_dashboard_set_remote_access')
                        ],
                        admin_url('admin-ajax.php')
                    )
                )
                ); ?>" method="post">

                    <label<?php if ($remoteStatus['status']) echo(' style="display:none;"'); ?>>
                        <?php esc_html_e('Email address', 'ags-dashboard'); ?>
                        <?php
                        $account = self::getCurrentUserAccount();
						if ($account->isLoggedIn()) {
							$defaultEmail = $account->getAccountEmail();
						}
                        ?>
                        <input type="email" name="remoteAccessEmail"<?php if (isset($defaultEmail)) echo('value="' . esc_attr($defaultEmail) . '"'); ?>>
                    </label>
                    <button class="ags-dashboard-btn-primary" value="<?php echo((int)!$remoteStatus['status']); ?>"><?php $remoteStatus['status'] ? esc_html_e('Disable Support Access', 'ags-dashboard') : esc_html_e('Enable Support Access', 'ags-dashboard'); ?></button>
                    <p class="agsd-description helper"><?php esc_html_e('Enter the email address you used or will be using to contact support', 'ags-dashboard'); ?></p>

                </form>

            </div>
        </div>

    <?php } ?>

</div>