<?php
AGSDashboard::VERSION; // Access control

class AGSDashboardApi {
	
	const METHOD_GET = 0;
	const METHOD_POST = 1;
	const METHOD_POST_JSON = 2;
	static $ENDPOINTS;
	private $account;
	
	public function __construct($account) {
		$this->account = $account;
	}
	
	public function __call($endpoint, $args) {
		
		if (!isset(self::$ENDPOINTS)) {
			self::$ENDPOINTS = array(
				'get_auth_token' => array( self::METHOD_POST, array('authTokenEmail', 'authTokenPassword', 'authTokenSite') ),
				'cancel_auth_token' => array( self::METHOD_POST, array() ),
				'get_license_keys' => array( self::METHOD_POST, array() ),
				'get_download_url' => array( self::METHOD_POST, array('licenseKey', 'downloadId') ),
				'query_help_articles' => array( self::METHOD_POST_JSON, array('request') ),
				'check_license_keys' => array( self::METHOD_POST_JSON, array('request') ),
				'register_remote_access_token' => array( self::METHOD_POST, array('url', 'email', 'token', 'expiry') ),
				'delete_remote_access_token' => array( self::METHOD_POST, array('url', 'token') ),
			);
		}
		
		if ( !isset( self::$ENDPOINTS[$endpoint] ) ) {
            throw new Exception( sprintf( __('Invalid endpoint name: %s', 'ags-dashboard'), $endpoint ) );
		}
		
		if (
			( !isset( $args[0] ) || !is_array($args[0]) || array_diff( array_keys($args[0]), self::$ENDPOINTS[$endpoint][1] ) )
			&& ( $args || self::$ENDPOINTS[$endpoint][1]  )
		) {
            throw new Exception( __('Missing or invalid arguments', 'ags-dashboard') );
		}
		
		$args = $args ? $args[0] : array();
		$args['action'] = 'ags_dashboard_'.$endpoint;
		$args['_ags_dashboard_ver'] = AGSDashboard::VERSION;
		
		if ( $endpoint != 'get_auth_token' ) {
			$token = $this->account->getToken();
			
			if (!$token) {
				if ( $endpoint == 'query_help_articles' || $endpoint == 'register_remote_access_token' || $endpoint == 'delete_remote_access_token' ) {
					$token = 'none';
				} else {
					throw new Exception();
				}
			}
			
			$args['_ags_dashboard_token'] = $token;
			$args['_ags_dashboard_site'] = get_option('siteurl');
		}
		
		switch ( self::$ENDPOINTS[$endpoint][0] ) {
			case self::METHOD_POST:
				$result = wp_remote_post( AGSDashboard::SITE_URL.'wp-admin/admin-ajax.php', array(
					'body' => $args,
					'timeout' => 300
				) );
				break;
			case self::METHOD_POST_JSON:
				$request = $args['request'];
				unset($args['request']);
				$result = wp_remote_post( AGSDashboard::SITE_URL.'wp-admin/admin-ajax.php?'.build_query($args), array(
					'headers' => 'Content-Type: application/json',
					'body' => wp_json_encode( $request ),
					'timeout' => 300
				) );
				break;
			case self::METHOD_GET:
				$result = wp_remote_get( AGSDashboard::SITE_URL.'wp-admin/admin-ajax.php?'.build_query($args), array(
					'timeout' => 300
				) );
				break;
		}
		
		$resultBody = wp_remote_retrieve_body($result);
		
		$resultBody = @json_decode($resultBody, true);
		
		return $resultBody;
		
	}
	
}