jQuery(document).ready(function($) {
	
	var catalogFiltering = {
		categories: [],
		tags: [],
		statuses: [],
		search: ''
	};

	function processItemClasses($item, classRemovePrefix, classAdd) {
		if (classRemovePrefix) {
			var currentClasses = $item.attr('class');
			if (currentClasses) {
				currentClasses = currentClasses.split(' ');
			} else {
				currentClasses = [];
			}
			for (var i = 0; i < currentClasses.length; ++i) {
				if ( currentClasses[i].substr( 0, classRemovePrefix.length) === classRemovePrefix ) {
					$item.removeClass( currentClasses[i] );
				}
			}
		}

		if (classAdd) {
			$item.addClass( classAdd );
		}

	}

	$('.ags-dashboard-settings-wrapper').on('click', '.ags-dashboard-ajax-link', function() {
		var $link = $(this),
			statusElemSelector = $link.data('ags-dashboard-status-elem'),
			statusTextElemSelector = $link.data('ags-dashboard-status-text-elem'),
			$status = statusElemSelector ? $link.closest('.ags-dashboard-catalog-item').find(statusElemSelector + ':first') : null,
			$statusText = statusTextElemSelector ? $link.closest('.ags-dashboard-catalog-item').find(statusTextElemSelector + ':first') : null;

		if ($link.hasClass('ags-dashboard-waiting')) {
			return;
		}

		$link.addClass('ags-dashboard-waiting');

		function showErrorMessage(message) {
			if (!message) {
				message = wp.i18n.__( 'Something went wrong; please try again.', 'ags-dashboard' );
			}

			alert(message);

			//location.reload();

			$link.removeClass('ags-dashboard-waiting');
		}

		var href = $link.data('ags-dashboard-href');
		if (!href) {
			href = $link.attr('href')
		}

		$.get(
			href,
			function(response) {
				if (response.success && response.data) {
					
					if (response.data.error) {
						showErrorMessage(response.data.error);
					} else {
						$link.removeClass('ags-dashboard-waiting');
					}

					if ($status) {
						processItemClasses($status, response.data.statusClassPrefix, response.data.statusClass);
					}
					if ($statusText) {
						$statusText.text( response.data.status ? response.data.status : wp.i18n.__( 'Unknown', 'ags-dashboard' ) );
					}

					if (response.data.reload) {
						location.reload();
						return;
					}

					if (response.data.redirect) {
						location.href = response.data.redirect;
						return;
					}
					
					if (response.data.message) {
						
						var buttons = {}, successDialog;
						
						if (response.data.messageLinks) {
							for ( label in response.data.messageLinks ) {
								buttons[label] = (function(url) {
									return function() {
										location.href = url;
									};
								})(response.data.messageLinks[label]);
							}
						}
						
						buttons[ wp.i18n.__( 'Close', 'ags-dashboard' ) ] = function() {
							successDialog.close();
						};
						
						successDialog = agsDashboardDialog.create({
							title: wp.i18n.__( 'Success', 'ags-dashboard' ),
							content: $('<p>').css('text-align', 'center').text(response.data.message),
							reverseButtonOrder: false,
							dialogClass: 'ags-layouts-ajax-success-dialog',
							buttonClass: 'ags-dashboard-btn',
							buttons: buttons
						});
						
					}
					
					if (response.data.productsCatalog) {
						var currentPage = $('#ags-dashboard-product-catalog .ags-dashboard-pager .ags-dashboard-active:first').data('ags-dashboard-page');
						$('#ags-dashboard-product-catalog').html( response.data.productsCatalog );
						initCatalogPaging( $('#ags-dashboard-product-catalog') );
						var $pageLink = $('#ags-dashboard-product-catalog .ags-dashboard-pager [data-ags-dashboard-page="' + currentPage + '"]:first');
						if (!$pageLink) {
							$pageLink = $('#ags-dashboard-product-catalog .ags-dashboard-pager a:last');
						}
						$pageLink.click();
					}

					if (!response.data.linkText && !response.data.linkHref) {
						$link.remove();
					} else if ( (!response.data.linkText || !response.data.linkHref) && !response.data.error) {
						$link.remove();
						showErrorMessage();
					} else {
						$link.attr('href', response.data.linkHref).text(response.data.linkText)
					}

					processItemClasses( $link.closest('.ags-dashboard-catalog-item'), response.data.rowClassRemovePrefix, response.data.rowClassAdd );

				} else {
					if ($statusText) {
						$statusText.text( wp.i18n.__( 'Unknown', 'ags-dashboard' ) );
					}
					showErrorMessage(response.data);
				}
			},
			'json'
		).fail(function() {
			showErrorMessage();
			if ($statusText) {
				$statusText.text( wp.i18n.__( 'Unknown', 'ags-dashboard' ) );
			}
		});

		return false;
	});
	
	$('#ags-dashboard-support-access form').submit(function() {

		var $form = $(this), $button = $form.find('button');

		if ($button.hasClass('ags-dashboard-waiting')) {
			return;
		}

		$button.addClass('ags-dashboard-waiting');
		$form.children('.ags-dashboard-error').remove();

		function showErrorMessage(message) {
			if (!message) {
				message = wp.i18n.__( 'Error updating support access settings; please refresh and try again.', 'ags-dashboard' );
			}

			$form.prepend(
				$('<p>')
					.addClass('ags-dashboard-error')
					.text( message )
			);

			$button.removeClass('ags-dashboard-waiting');
		}

		$.post(
			$form.attr('action'),
			$form.serialize() + '&remoteAccessActive=' + $button.val(),
			function(response) {
				if (response.success && response.data) {
					setSupportAccessStatus(response.data.status, response.data.statusText);
					$button.removeClass('ags-dashboard-waiting');
				} else {
					showErrorMessage(response.data);
				}
			},
			'json'
		).fail(function() {
			showErrorMessage();
		});

		return false;
	});
	
	function setSupportAccessStatus(isEnabled, statusText) {
		$('#ags-dashboard-support-access-status')
			.removeClass('ags-dashboard-support-access-enabled ags-dashboard-support-access-disabled')
			.addClass('ags-dashboard-support-access-' + ( isEnabled ? 'enabled' : 'disabled' ))
			.text( statusText );
			
		var $button = $('#ags-dashboard-support-access button:first');
		
		$button
			.val( isEnabled ? 0 : 1)
			.text( isEnabled ? wp.i18n.__('Disable Support Access', 'ags-dashboard') : wp.i18n.__('Enable Support Access', 'ags-dashboard') );
		$button.siblings('label').toggle( !isEnabled );
	}


	$('#ags-dashboard-installed-products').on('change', '.ags-dashboard-license-key-select', function() {
		var $select = $(this), selectValue = $select.val();
		if ( selectValue ) {
			$select.siblings('input[name="licenseKey"]:first').val( selectValue ).change();
		}
	});
	
	var supportArticlesSearchTimeout;
	$('#ags-dashboard-support-articles input[type="search"]').on('input', function() {
		
		if (supportArticlesSearchTimeout) {
			clearTimeout(supportArticlesSearchTimeout);
		}
		var query = $(this).val();
		
		function doUpdate() {
			supportArticlesSearchTimeout = null;
			
			var $container = $('#ags-dashboard-support-articles-list-container');
			var $loader = $container.children('.ags-dashboard-loading').show();
			$container.children(':not(.ags-dashboard-loading)').remove();

			function handleError() {
				$loader.hide();
			}

			var request = {
				action: 'ags_dashboard_get_support_articles'
			};

			if (query) {
				request.q = query;
			}

			$.post(
				ajaxurl,
				request,
				function(response) {
					$container.children(':not(.ags-dashboard-loading)').remove();
					
					if (response.success && response.data) {
						if (response.data.length) {
							var $articlesList = $('<ul>').addClass('ags-dashboard-support-articles-list');
							for ( var i = 0; i < response.data.length; ++i ) {
								if (response.data[i] && response.data[i].name && response.data[i].url) {
									var $articleItem = $('<li>').append(
										$('<a>')
											.addClass('ags-dashboard-support-article-title')
											.attr({
												href: response.data[i].url,
												target: '_blank'
											})
											.text(response.data[i].name)
									);

									if ( response.data[i].excerpt ) {
										$articleItem.append(
											$('<span>')
												.addClass('ags-dashboard-support-article-excerpt')
												.text(response.data[i].excerpt)
										);
									}

									$articleItem.appendTo( $articlesList );
								}
							}
							$articlesList.appendTo($container);
						} else {
							$('<p>').addClass('ags-dashboard-nothing').text( wp.i18n.__('No answers found') ).appendTo($container);
						}
						
						$loader.hide();
						
					} else {
						handleError();
					}
				},
				'json'
			).fail(handleError);
		}
		
		if (query) {
			supportArticlesSearchTimeout = setTimeout(doUpdate, 500);
		} else {
			doUpdate();
		}
		


	}).trigger('input');
	
	function setCatalogFiltering() {
		$('#ags-dashboard-product-catalog-filter-style').remove();
		
		var css = '';
		if ( catalogFiltering.categories.length ) {
			css += '#ags-dashboard-product-catalog > div';
			for ( var i = 0; i < catalogFiltering.categories.length; ++i ) {
				css += ':not(.ags-dashoard-catalog-item-category-' + catalogFiltering.categories[i] + ')';
			}
			css += '{ display: none; }';
		}
		if ( catalogFiltering.tags.length ) {
			css += '#ags-dashboard-product-catalog > div';
			for ( var i = 0; i < catalogFiltering.tags.length; ++i ) {
				css += ':not(.ags-dashoard-catalog-item-tag-' + catalogFiltering.tags[i] + ')';
			}
			css += '{ display: none; }';
		}
		if ( catalogFiltering.statuses.length ) {
			css += '#ags-dashboard-product-catalog > div';
			for ( var i = 0; i < catalogFiltering.statuses.length; ++i ) {
				css += ':not(.ags-dashoard-catalog-item-status-' + catalogFiltering.statuses[i] + ')';
			}
			css += '{ display: none; }';
		}
		if ( catalogFiltering.search.length ) {
			var searchWords = catalogFiltering.search.trim().toLowerCase().split(' ');
			if (searchWords.length) {
				var firstWord = true;
				for ( var i = 0; i < searchWords.length; ++i ) {
					if (searchWords[i]) {
						css += (firstWord ? '' : ', ') + '#ags-dashboard-product-catalog > div:not([data-ags-dashboard-item-name~="' + searchWords[i].trim() + '"])';
						firstWord = false;
					}
				}
				css += '{ display: none; }';
			}
		}
		
		if (css) {
			var $filterStyle = $('<style>')
									.attr('id', 'ags-dashboard-product-catalog-filter-style')
									.text(css);
			$filterStyle.appendTo('head');
		}
		initCatalogPaging( $('#ags-dashboard-product-catalog') );
	}

	$('#ags-dashboard-product-catalog-filtering a').click(function() {
		var $filterLink = $(this), $filterLinkLi = $filterLink.closest('li'), category = $filterLink.data('ags-dashboard-category');
		
		catalogFiltering.categories = category && category !== '*' ? [ category ] : [];
		setCatalogFiltering();

		$filterLinkLi.siblings().removeClass('ags-dashboard-active');
		$filterLinkLi.addClass('ags-dashboard-active');
		
		$('#ags-dashboard-product-catalog-filtering-dropdown input[value="' + category + '"]:first').prop('checked', true);

		return false;
	});
	
	$('#ags-dashboard-product-catalog-filtering-dropdown input').change(function() {
		$('#ags-dashboard-product-catalog-filtering a[data-ags-dashboard-category="' + $(this).val() + '"]:first').click();
	});
	

	$('#ags-dashboard-product-catalog-status-options a').click(function() {
		var $filterLink = $(this), $filterLinkLi = $filterLink.closest('li'), status = $filterLink.data('ags-dashboard-status');
		
		if ( $filterLinkLi.hasClass('ags-dashboard-active') ) {
			var currentIndex = catalogFiltering.statuses.indexOf( status );
			if (currentIndex !== -1) {
				catalogFiltering.statuses.splice(currentIndex, 1);
			}
			$filterLinkLi.removeClass('ags-dashboard-active');
			$('#ags-dashboard-product-catalog-status-options-dropdown input[value="' + status + '"]:first').prop('checked', false);
		} else {
			catalogFiltering.statuses.push( status );
			$filterLinkLi.addClass('ags-dashboard-active');
			$('#ags-dashboard-product-catalog-status-options-dropdown input[value="' + status + '"]:first').prop('checked', true);
		}
		
		
		setCatalogFiltering();

		return false;
	});
	
	$('#ags-dashboard-product-catalog-status-options-dropdown input').change(function() {
		$('#ags-dashboard-product-catalog-status-options a[data-ags-dashboard-status="' + $(this).val() + '"]:first').click();
	});
	
	
	
	$('#ags-dashboard-product-catalog-tag-options input').change(function() {
		var $filterCheckbox = $(this);
		
		if ($filterCheckbox.is(':checked')) {
			catalogFiltering.tags.push( $filterCheckbox.val() );
		} else {
			var currentIndex = catalogFiltering.tags.indexOf( $filterCheckbox.val() );
			if (currentIndex !== -1) {
				catalogFiltering.tags.splice(currentIndex, 1);
			}
		}
		
		setCatalogFiltering();

		return false;
	});
	
	var searchTimeout;
	
	$('#ags-dashboard-product-catalog-search').on('input', function() {
		if (searchTimeout) {
			clearTimeout(searchTimeout);
		}
		var searchField = this;
		searchTimeout = setTimeout(function() {
			searchTimeout = null;
			catalogFiltering.search = $(searchField).val();
			setCatalogFiltering();
			return false;
		}, 500);
	});
	
	$('#ags-dashboard-product-catalog-search[value]').trigger('input');

	function initCatalogPaging($catalog) {
		$catalog.children('.ags-dashboard-pager, .ags-dashboard-nothing').remove();
		$('#ags-dashboard-product-catalog-paging-style').remove();

		var perPage = $catalog.data('ags-dashboard-per-page'),
			$catalogChildren = $catalog.children(),
			$visibleItems = $catalogChildren.filter('.ags-dashboard-catalog-item:visible'),
			numPages = Math.ceil($visibleItems.length / perPage),
			currentPage = 1;
		
			if (numPages) {

			var $pager = $('<ul>').addClass('ags-dashboard-pager');

			$('<li>')
				.append(
					$('<a>')
						.attr({
							href: '#',
							'data-ags-dashboard-page': '-'
						})
						.text( wp.i18n.__( 'Previous', 'ags-dashboard' ) )
				)
				.appendTo($pager);


			for (var i = 0; i < numPages; ++i) {
				$('<li>')
					.append(
						$('<a>')
							.attr({
								href: '#',
								'data-ags-dashboard-page': i + 1
							})
							.text(i + 1)
					)
					.appendTo($pager);
			}

			$('<li>')
				.append(
					$('<a>')
						.attr({
							href: '#',
							'data-ags-dashboard-page': '+'
						})
						.text( wp.i18n.__( 'Next', 'ags-dashboard' ) )
				)
				.appendTo($pager);

			function setPage(pageNum) {
				pageNum = parseInt(pageNum);
				
				$('#ags-dashboard-product-catalog-paging-style').remove();

				var visibleSelectors = '';
				for ( var i = (pageNum - 1) * perPage; i < pageNum * perPage; ++i ) {
					visibleSelectors += (visibleSelectors ? ',' : '') + '#ags-dashboard-product-catalog .ags-dashboard-catalog-item:nth-child(' + ( $catalogChildren.index( $visibleItems.eq(i) ) + 1 ) + ')';
				}

				var $pagingStyle = $('<style>')
										.attr('id', 'ags-dashboard-product-catalog-paging-style')
										.text(
											'#ags-dashboard-product-catalog .ags-dashboard-catalog-item { display: none; } '
											+ visibleSelectors + ' { display: block; }'
										);

				$pagingStyle.appendTo('head');

				currentPage = pageNum;

				$pager.find('.ags-dashboard-active').removeClass('ags-dashboard-active');
				$pager.find('[data-ags-dashboard-page="' + pageNum + '"]').addClass('ags-dashboard-active');

				$pager.find('[data-ags-dashboard-page="-"]').toggle(pageNum !== 1);
				$pager.find('[data-ags-dashboard-page="+"]').toggle(pageNum !== numPages);

			}

			$pager.on('click', 'a', function() {
				var page = $(this).data('ags-dashboard-page');
				switch ( page ) {
					case '+':
						setPage(currentPage + 1);
						break;
					case '-':
						setPage(currentPage - 1);
						break;
					default:
						setPage(page);
				}
				return false;
			});

			$catalog.append($pager);

			setPage(1);
			
		} else { // No pages
			$('<p>').addClass('ags-dashboard-nothing').text(wp.i18n.__('There are no items to display.')).appendTo($catalog);
		}
	}

	initCatalogPaging( $('#ags-dashboard-product-catalog') );

	$('#ags-dashboard-settings-preferences :input').change(function() {

		var $input = $(this);
		$input.attr('disabled', true);

		$('#ags-dashboard-settings-preferences .ags-dashboard-success, #ags-dashboard-settings-preferences .ags-dashboard-error').hide();

		if ($input.is(':checkbox')) {
			var value = $input.is(':checked') ? 1 : 0;
		} else {
			var value = $input.val();
		}

		function handleError() {
			$('#ags-dashboard-settings-preferences .ags-dashboard-error').fadeIn();
			$input.attr('disabled', false);
		}

		$.post(
			ajaxurl,
			{
				action: 'ags_dashboard_set_user_preference',
				nonce: $('#ags-dashboard-change-user-prefs-nonce').val(),
				pref: $input.attr('name'),
				value: value
			},
			function(response) {
				if (response.success) {
					$('#ags-dashboard-settings-preferences .ags-dashboard-success').fadeIn().delay(3000).fadeOut();
					$input.attr('disabled', false);
				} else {
					handleError();
				}
			},
			'json'
		).fail(handleError);
	});
	
	
	$('.ags-dashboard-settings-wrapper').on('click', '.ags-dashboard-dropdown a', function() {
		var $dropdown = $(this).closest('.ags-dashboard-dropdown');
		var wasOpen = $dropdown.hasClass('open');
		$('.ags-dashboard-dropdown.open').removeClass('open');
		if (!wasOpen) {
			$dropdown.addClass('open');
		}
		
		if ($(this).attr('href') === '#') {
			return false;
		}
	});
	
	$('.ags-dashboard-settings-wrapper').on('click', '.ags-dashboard-purchase-link[href="#"]', function() {
		agsDashboardDialog.create({
			content: $(this).siblings('.ags-dashboard-purchase-options').html(),
			dialogClass: 'ags-dashboard-purchase-options-dialog'
		});
		
		return false;
	});

	$('#ags-dashboard-admin-navbar').on('click', '.ags-dashboard-dropdown span', function() {
		$(this).closest('.ags-dashboard-dropdown').toggleClass('open');
		return false;
	});
	
	$('#ags-dashboard-product-catalog').on('click', '.ags-dashboard-manage-license-key-link', function() {
		var dialog, $item = $(this).closest('.ags-dashboard-catalog-item'), $manageLicenseKeyLink = $item.find('a.ags-dashboard-manage-license-key-link:first');
		
		if (!$manageLicenseKeyLink.length) {
			return false;
		}
		
		dialog = agsDashboardDialog.create({
			title: wp.i18n.__( 'Manage License Key', 'ags-dashboard' ),
			reverseButtonOrder: false,
			dialogClass: 'ags-layouts-license-key-dialog',
			firstButtonClass: 'ags-dashboard-btn',
			buttonClass: 'ags-dashboard-btn-primary'
		});

		dialog.showLoader();


		$.get(
			$manageLicenseKeyLink.attr('href'),
			null,
			function(response) {
				if (response.success && response.data && response.data.formHtml) {
					var $dialogContent = dialog.element.find('.ags-dashboard-dialog-content').html(response.data.formHtml);
					
									

					$dialogContent.on('click', '.ags-dashboard-license-key-dropdown ul a', function() {
						$(this).closest('form').find('input:first').val( $(this).children('.ags-dashboard-license-key-select-key:first').text() );
						// don't return false here - the next event handler will take care of that
					});
					
					
					$dialogContent.on('click', '.ags-dashboard-dropdown a', function() {
						$(this).closest('.ags-dashboard-dropdown').toggleClass('open');
						return false;
					});


					var dialogButtons = {};
					
					dialogButtons[ wp.i18n.__( 'Close', 'ags-dashboard' ) ] = function() {
						dialog.close();
					};
					
					if ( response.data.buttonText && typeof response.data.buttonValue !== 'undefined' ) {
						var buttonText = response.data.buttonText, buttonValue = response.data.buttonValue;
						dialogButtons[ buttonText ] = function() {
							dialog.showLoader();
							
							//dialog.setButtons({});
							
							var $form = $dialogContent.find('form');
							
							$form.children('.ags-dashboard-error').remove();

							function showErrorMessage(message) {
								if (!message) {
									message = wp.i18n.__( 'Error updating license key activation; please refresh and try again.', 'ags-dashboard' );
								}

								$form.prepend(
									$('<p>')
										.addClass('ags-dashboard-error')
										.text( message )
								);

								dialog.hideLoader();
							}
							$.post(
								$form.attr('action'),
								$form.serialize() + '&activate=' + buttonValue,
								function(response) {
									if (response.success && response.data.formHtml) {
										$dialogContent.html(response.data.formHtml);
										$form = $dialogContent.find('form');
										if ( response.data.buttonText && typeof response.data.buttonValue !== 'undefined' ) {
											buttonValue = response.data.buttonValue;
											if (response.data.buttonText !== buttonText) {
												dialogButtons[response.data.buttonText] = dialogButtons[buttonText];
												delete dialogButtons[buttonText];
												buttonText = response.data.buttonText;
											}
											dialog.setButtons(dialogButtons);
										}
										processItemClasses( $item, response.data.rowClassRemovePrefix, response.data.rowClassAdd );
										
										var $status = $item.find('.ags-dashboard-product-status-key:first');
										
										$status.removeClass('ags-dashboard-active ags-dashboard-warning ags-dashboard-error');
										$status.children('.ags-dashboard-tooltip-text:first').text(response.data.status);
										if (response.data.statusClass) {
											$status.addClass(response.data.statusClass);
										}
										
										if (response.data.error) {
											showErrorMessage(response.data.error);
										}
										
										dialog.hideLoader();
									} else {
										showErrorMessage(response.data);
									}
								},
								'json'
							).fail(function() {
								showErrorMessage();
							});
							
						};
					}
					
					dialog.setButtons(dialogButtons);
					dialog.hideLoader();

				}
			}
		);
		
		return false;
	});
	
	$('#ags-dashboard-system-info-link').click(function() {
		var dialog;
		
		var dialogButtons = {};
		dialogButtons[ wp.i18n.__( 'Close', 'ags-dashboard' ) ] = function() {
			dialog.close();
		};
		
		dialog = agsDashboardDialog.create({
			title: wp.i18n.__( 'System Info', 'ags-dashboard' ),
			content: $('<pre>').text( $('#ags-dashboard-system-info').text() ),
			dialogClass: 'ags-dashboard-system-info-dialog',
			firstButtonClass: 'ags-dashboard-btn-primary',
			buttons: dialogButtons
		});
		
		return false;
	});
	
	$('#ags-dashboard-support-ticket-form').submit(function() {
		
		var $form = $(this), $button = $form.find('button'), $supportAccessButton = $('#ags-dashboard-support-access button:first');

		if ($button.hasClass('ags-dashboard-waiting')) {
			return;
		}
		
		$supportAccessButton.attr('disabled', true);

		$button.addClass('ags-dashboard-waiting');
		$form.children('.ags-dashboard-error, .ags-dashboard-success').remove();

		function showErrorMessage(message) {
			if (!message) {
				message = wp.i18n.__( 'Error opening ticket; please refresh and try again.', 'ags-dashboard' );
			}

			$form.prepend(
				$('<p>')
					.addClass('ags-dashboard-error')
					.text( message )
			);

			$button.removeClass('ags-dashboard-waiting');
			$supportAccessButton.attr('disabled', false);
		}

		$.post(
			$form.attr('action'),
			$form.serialize(),
			function(response) {
				if (response.success && response.data && response.data.remoteAccessStatus) {
					
					$form.prepend(
						$('<p>')
							.addClass('ags-dashboard-success')
							.text( wp.i18n.__( 'Your ticket was submitted successfully. You should receive a response by email soon.', 'ags-dashboard' ) )
					);
					
					setSupportAccessStatus(response.data.remoteAccessStatus.status, response.data.remoteAccessStatus.statusText);
					
					$button.removeClass('ags-dashboard-waiting');
					$supportAccessButton.attr('disabled', false);
					
					$form.find('input[type="text"], input[type="email"], textarea:visible').val('');

				} else {
					showErrorMessage(response.data);
				}
			},
			'json'
		).fail(function() {
			showErrorMessage();
		});
		
		
		return false;
	});
	
	$('.ags-dashboard-settings-wrapper').on('click', '.ags-dashboard-refer-network-admin', function() {
		alert( wp.i18n.__( 'This feature is only available in the Network Admin for your multisite installation. Please go to WP Zone Helper in your Network Admin or contact your multisite network administrator.', 'ags-dashboard' ) );
		return false;
	});

	$('#ags-dashboard-wizard').each(function() {

		function loadWizardStep(step, validate) {
			var requestData = $('#ags-dashboard-wizard form').serializeArray();
			
			if (validate) {
				// No empty values allowed
				for (var i = 0; i < requestData.length; ++i) {
					if (!requestData[i].value.length) {
						alert('One or more fields are empty. Please complete all fields and try again.');
						
						return;
					}
				}
			}
			
			$('#ags-dashboard-wizard-loading').show();


			requestData.push({
				name: 'action',
				value: 'ags_dashboard_wizard'
			});
			requestData.push({
				name: 'step',
				value: step
			});


			$.post(
				ajaxurl,
				requestData,
				function(response) {
					if (response.success && response.data && response.data.wizardHtml) {
						$('#ags-dashboard-wizard-content').html(response.data.wizardHtml);
						
						setupButtons(step);

						$('#ags-dashboard-wizard-steps > li').each(function(i) {
							$(this).toggleClass('ags-dashboard-wizard-step-future', i + 1 > step);
						});
						
						$('#ags-dashboard-wizard-loading').hide();

					}
				}
			);
		}
		
		function setupButtons(step) {
			var $buttonPrimary = $('#ags-dashboard-wizard-buttons > :last-child'), $buttonSecondary = $('#ags-dashboard-wizard-buttons > :first-child');
			
			if (step === 3) {
				$buttonPrimary
					.text( wp.i18n.__( 'Finish', 'ags-dashboard' ) )
					.off('click')
					.click(function() {
						location.href = location.href.replace('&wizard=1', '');
					});
			} else {
				$buttonPrimary
					.text( step === 1 ? wp.i18n.__( 'Continue', 'ags-dashboard' ) : wp.i18n.__( 'Log in', 'ags-dashboard' ) )
					.off('click')
					.click(function() {
						loadWizardStep(step + 1, true);
					});
			}

			if (step === 1) {
				$buttonSecondary
					.text( wp.i18n.__( 'Skip', 'ags-dashboard' ) )
					.off('click')
					.click(function() {
						location.href = location.href.replace('&wizard=1', '');
					});
			} else {
				$buttonSecondary
					.text( wp.i18n.__( 'Back', 'ags-dashboard' ) )
					.off('click')
					.click(function() {
						loadWizardStep(step - 1);
					});
			}

		}

		setupButtons(1);
	});

});